from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class ReactivePowerUnits(Enum):
        """
            ReactivePowerUnits enumeration
        """
        
        VoltampereReactive = 'VoltampereReactive'
        """
            
        """
        
        KilovoltampereReactive = 'KilovoltampereReactive'
        """
            
        """
        
        MegavoltampereReactive = 'MegavoltampereReactive'
        """
            
        """
        
        GigavoltampereReactive = 'GigavoltampereReactive'
        """
            
        """
        

class ReactivePowerDto:
    """
    A DTO representation of a ReactivePower

    Attributes:
        value (float): The value of the ReactivePower.
        unit (ReactivePowerUnits): The specific unit that the ReactivePower value is representing.
    """

    def __init__(self, value: float, unit: ReactivePowerUnits):
        """
        Create a new DTO representation of a ReactivePower

        Parameters:
            value (float): The value of the ReactivePower.
            unit (ReactivePowerUnits): The specific unit that the ReactivePower value is representing.
        """
        self.value: float = value
        """
        The value of the ReactivePower
        """
        self.unit: ReactivePowerUnits = unit
        """
        The specific unit that the ReactivePower value is representing
        """

    def to_json(self):
        """
        Get a ReactivePower DTO JSON object representing the current unit.

        :return: JSON object represents ReactivePower DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "VoltampereReactive"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of ReactivePower DTO from a json representation.

        :param data: The ReactivePower DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "VoltampereReactive"}
        :return: A new instance of ReactivePowerDto.
        :rtype: ReactivePowerDto
        """
        return ReactivePowerDto(value=data["value"], unit=ReactivePowerUnits(data["unit"]))


class ReactivePower(AbstractMeasure):
    """
    Volt-ampere reactive (var) is a unit by which reactive power is expressed in an AC electric power system. Reactive power exists in an AC circuit when the current and voltage are not in phase.

    Args:
        value (float): The value.
        from_unit (ReactivePowerUnits): The ReactivePower unit to create from, The default unit is VoltampereReactive
    """
    def __init__(self, value: float, from_unit: ReactivePowerUnits = ReactivePowerUnits.VoltampereReactive):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__voltamperes_reactive = None
        
        self.__kilovoltamperes_reactive = None
        
        self.__megavoltamperes_reactive = None
        
        self.__gigavoltamperes_reactive = None
        

    def convert(self, unit: ReactivePowerUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: ReactivePowerUnits = ReactivePowerUnits.VoltampereReactive) -> ReactivePowerDto:
        """
        Get a new instance of ReactivePower DTO representing the current unit.

        :param hold_in_unit: The specific ReactivePower unit to store the ReactivePower value in the DTO representation.
        :type hold_in_unit: ReactivePowerUnits
        :return: A new instance of ReactivePowerDto.
        :rtype: ReactivePowerDto
        """
        return ReactivePowerDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: ReactivePowerUnits = ReactivePowerUnits.VoltampereReactive):
        """
        Get a ReactivePower DTO JSON object representing the current unit.

        :param hold_in_unit: The specific ReactivePower unit to store the ReactivePower value in the DTO representation.
        :type hold_in_unit: ReactivePowerUnits
        :return: JSON object represents ReactivePower DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "VoltampereReactive"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(reactive_power_dto: ReactivePowerDto):
        """
        Obtain a new instance of ReactivePower from a DTO unit object.

        :param reactive_power_dto: The ReactivePower DTO representation.
        :type reactive_power_dto: ReactivePowerDto
        :return: A new instance of ReactivePower.
        :rtype: ReactivePower
        """
        return ReactivePower(reactive_power_dto.value, reactive_power_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of ReactivePower from a DTO unit json representation.

        :param data: The ReactivePower DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "VoltampereReactive"}
        :return: A new instance of ReactivePower.
        :rtype: ReactivePower
        """
        return ReactivePower.from_dto(ReactivePowerDto.from_json(data))

    def __convert_from_base(self, from_unit: ReactivePowerUnits) -> float:
        value = self._value
        
        if from_unit == ReactivePowerUnits.VoltampereReactive:
            return (value)
        
        if from_unit == ReactivePowerUnits.KilovoltampereReactive:
            return ((value) / 1000.0)
        
        if from_unit == ReactivePowerUnits.MegavoltampereReactive:
            return ((value) / 1000000.0)
        
        if from_unit == ReactivePowerUnits.GigavoltampereReactive:
            return ((value) / 1000000000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: ReactivePowerUnits) -> float:
        
        if to_unit == ReactivePowerUnits.VoltampereReactive:
            return (value)
        
        if to_unit == ReactivePowerUnits.KilovoltampereReactive:
            return ((value) * 1000.0)
        
        if to_unit == ReactivePowerUnits.MegavoltampereReactive:
            return ((value) * 1000000.0)
        
        if to_unit == ReactivePowerUnits.GigavoltampereReactive:
            return ((value) * 1000000000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_voltamperes_reactive(voltamperes_reactive: float):
        """
        Create a new instance of ReactivePower from a value in voltamperes_reactive.

        

        :param meters: The ReactivePower value in voltamperes_reactive.
        :type voltamperes_reactive: float
        :return: A new instance of ReactivePower.
        :rtype: ReactivePower
        """
        return ReactivePower(voltamperes_reactive, ReactivePowerUnits.VoltampereReactive)

    
    @staticmethod
    def from_kilovoltamperes_reactive(kilovoltamperes_reactive: float):
        """
        Create a new instance of ReactivePower from a value in kilovoltamperes_reactive.

        

        :param meters: The ReactivePower value in kilovoltamperes_reactive.
        :type kilovoltamperes_reactive: float
        :return: A new instance of ReactivePower.
        :rtype: ReactivePower
        """
        return ReactivePower(kilovoltamperes_reactive, ReactivePowerUnits.KilovoltampereReactive)

    
    @staticmethod
    def from_megavoltamperes_reactive(megavoltamperes_reactive: float):
        """
        Create a new instance of ReactivePower from a value in megavoltamperes_reactive.

        

        :param meters: The ReactivePower value in megavoltamperes_reactive.
        :type megavoltamperes_reactive: float
        :return: A new instance of ReactivePower.
        :rtype: ReactivePower
        """
        return ReactivePower(megavoltamperes_reactive, ReactivePowerUnits.MegavoltampereReactive)

    
    @staticmethod
    def from_gigavoltamperes_reactive(gigavoltamperes_reactive: float):
        """
        Create a new instance of ReactivePower from a value in gigavoltamperes_reactive.

        

        :param meters: The ReactivePower value in gigavoltamperes_reactive.
        :type gigavoltamperes_reactive: float
        :return: A new instance of ReactivePower.
        :rtype: ReactivePower
        """
        return ReactivePower(gigavoltamperes_reactive, ReactivePowerUnits.GigavoltampereReactive)

    
    @property
    def voltamperes_reactive(self) -> float:
        """
        
        """
        if self.__voltamperes_reactive != None:
            return self.__voltamperes_reactive
        self.__voltamperes_reactive = self.__convert_from_base(ReactivePowerUnits.VoltampereReactive)
        return self.__voltamperes_reactive

    
    @property
    def kilovoltamperes_reactive(self) -> float:
        """
        
        """
        if self.__kilovoltamperes_reactive != None:
            return self.__kilovoltamperes_reactive
        self.__kilovoltamperes_reactive = self.__convert_from_base(ReactivePowerUnits.KilovoltampereReactive)
        return self.__kilovoltamperes_reactive

    
    @property
    def megavoltamperes_reactive(self) -> float:
        """
        
        """
        if self.__megavoltamperes_reactive != None:
            return self.__megavoltamperes_reactive
        self.__megavoltamperes_reactive = self.__convert_from_base(ReactivePowerUnits.MegavoltampereReactive)
        return self.__megavoltamperes_reactive

    
    @property
    def gigavoltamperes_reactive(self) -> float:
        """
        
        """
        if self.__gigavoltamperes_reactive != None:
            return self.__gigavoltamperes_reactive
        self.__gigavoltamperes_reactive = self.__convert_from_base(ReactivePowerUnits.GigavoltampereReactive)
        return self.__gigavoltamperes_reactive

    
    def to_string(self, unit: ReactivePowerUnits = ReactivePowerUnits.VoltampereReactive, fractional_digits: int = None) -> str:
        """
        Format the ReactivePower to a string.
        
        Note: the default format for ReactivePower is VoltampereReactive.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the ReactivePower. Default is 'VoltampereReactive'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == ReactivePowerUnits.VoltampereReactive:
            return f"""{super()._truncate_fraction_digits(self.voltamperes_reactive, fractional_digits)} var"""
        
        if unit == ReactivePowerUnits.KilovoltampereReactive:
            return f"""{super()._truncate_fraction_digits(self.kilovoltamperes_reactive, fractional_digits)} kvar"""
        
        if unit == ReactivePowerUnits.MegavoltampereReactive:
            return f"""{super()._truncate_fraction_digits(self.megavoltamperes_reactive, fractional_digits)} Mvar"""
        
        if unit == ReactivePowerUnits.GigavoltampereReactive:
            return f"""{super()._truncate_fraction_digits(self.gigavoltamperes_reactive, fractional_digits)} Gvar"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: ReactivePowerUnits = ReactivePowerUnits.VoltampereReactive) -> str:
        """
        Get ReactivePower unit abbreviation.
        Note! the default abbreviation for ReactivePower is VoltampereReactive.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == ReactivePowerUnits.VoltampereReactive:
            return """var"""
        
        if unit_abbreviation == ReactivePowerUnits.KilovoltampereReactive:
            return """kvar"""
        
        if unit_abbreviation == ReactivePowerUnits.MegavoltampereReactive:
            return """Mvar"""
        
        if unit_abbreviation == ReactivePowerUnits.GigavoltampereReactive:
            return """Gvar"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class RadiationEquivalentDoseUnits(Enum):
        """
            RadiationEquivalentDoseUnits enumeration
        """
        
        Sievert = 'Sievert'
        """
            The sievert is a unit in the International System of Units (SI) intended to represent the stochastic health risk of ionizing radiation, which is defined as the probability of causing radiation-induced cancer and genetic damage.
        """
        
        RoentgenEquivalentMan = 'RoentgenEquivalentMan'
        """
            
        """
        
        Nanosievert = 'Nanosievert'
        """
            
        """
        
        Microsievert = 'Microsievert'
        """
            
        """
        
        Millisievert = 'Millisievert'
        """
            
        """
        
        MilliroentgenEquivalentMan = 'MilliroentgenEquivalentMan'
        """
            
        """
        

class RadiationEquivalentDoseDto:
    """
    A DTO representation of a RadiationEquivalentDose

    Attributes:
        value (float): The value of the RadiationEquivalentDose.
        unit (RadiationEquivalentDoseUnits): The specific unit that the RadiationEquivalentDose value is representing.
    """

    def __init__(self, value: float, unit: RadiationEquivalentDoseUnits):
        """
        Create a new DTO representation of a RadiationEquivalentDose

        Parameters:
            value (float): The value of the RadiationEquivalentDose.
            unit (RadiationEquivalentDoseUnits): The specific unit that the RadiationEquivalentDose value is representing.
        """
        self.value: float = value
        """
        The value of the RadiationEquivalentDose
        """
        self.unit: RadiationEquivalentDoseUnits = unit
        """
        The specific unit that the RadiationEquivalentDose value is representing
        """

    def to_json(self):
        """
        Get a RadiationEquivalentDose DTO JSON object representing the current unit.

        :return: JSON object represents RadiationEquivalentDose DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "Sievert"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of RadiationEquivalentDose DTO from a json representation.

        :param data: The RadiationEquivalentDose DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "Sievert"}
        :return: A new instance of RadiationEquivalentDoseDto.
        :rtype: RadiationEquivalentDoseDto
        """
        return RadiationEquivalentDoseDto(value=data["value"], unit=RadiationEquivalentDoseUnits(data["unit"]))


class RadiationEquivalentDose(AbstractMeasure):
    """
    Equivalent dose is a dose quantity representing the stochastic health effects of low levels of ionizing radiation on the human body which represents the probability of radiation-induced cancer and genetic damage.

    Args:
        value (float): The value.
        from_unit (RadiationEquivalentDoseUnits): The RadiationEquivalentDose unit to create from, The default unit is Sievert
    """
    def __init__(self, value: float, from_unit: RadiationEquivalentDoseUnits = RadiationEquivalentDoseUnits.Sievert):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__sieverts = None
        
        self.__roentgens_equivalent_man = None
        
        self.__nanosieverts = None
        
        self.__microsieverts = None
        
        self.__millisieverts = None
        
        self.__milliroentgens_equivalent_man = None
        

    def convert(self, unit: RadiationEquivalentDoseUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: RadiationEquivalentDoseUnits = RadiationEquivalentDoseUnits.Sievert) -> RadiationEquivalentDoseDto:
        """
        Get a new instance of RadiationEquivalentDose DTO representing the current unit.

        :param hold_in_unit: The specific RadiationEquivalentDose unit to store the RadiationEquivalentDose value in the DTO representation.
        :type hold_in_unit: RadiationEquivalentDoseUnits
        :return: A new instance of RadiationEquivalentDoseDto.
        :rtype: RadiationEquivalentDoseDto
        """
        return RadiationEquivalentDoseDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: RadiationEquivalentDoseUnits = RadiationEquivalentDoseUnits.Sievert):
        """
        Get a RadiationEquivalentDose DTO JSON object representing the current unit.

        :param hold_in_unit: The specific RadiationEquivalentDose unit to store the RadiationEquivalentDose value in the DTO representation.
        :type hold_in_unit: RadiationEquivalentDoseUnits
        :return: JSON object represents RadiationEquivalentDose DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "Sievert"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(radiation_equivalent_dose_dto: RadiationEquivalentDoseDto):
        """
        Obtain a new instance of RadiationEquivalentDose from a DTO unit object.

        :param radiation_equivalent_dose_dto: The RadiationEquivalentDose DTO representation.
        :type radiation_equivalent_dose_dto: RadiationEquivalentDoseDto
        :return: A new instance of RadiationEquivalentDose.
        :rtype: RadiationEquivalentDose
        """
        return RadiationEquivalentDose(radiation_equivalent_dose_dto.value, radiation_equivalent_dose_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of RadiationEquivalentDose from a DTO unit json representation.

        :param data: The RadiationEquivalentDose DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "Sievert"}
        :return: A new instance of RadiationEquivalentDose.
        :rtype: RadiationEquivalentDose
        """
        return RadiationEquivalentDose.from_dto(RadiationEquivalentDoseDto.from_json(data))

    def __convert_from_base(self, from_unit: RadiationEquivalentDoseUnits) -> float:
        value = self._value
        
        if from_unit == RadiationEquivalentDoseUnits.Sievert:
            return (value)
        
        if from_unit == RadiationEquivalentDoseUnits.RoentgenEquivalentMan:
            return (value * 100)
        
        if from_unit == RadiationEquivalentDoseUnits.Nanosievert:
            return ((value) / 1e-09)
        
        if from_unit == RadiationEquivalentDoseUnits.Microsievert:
            return ((value) / 1e-06)
        
        if from_unit == RadiationEquivalentDoseUnits.Millisievert:
            return ((value) / 0.001)
        
        if from_unit == RadiationEquivalentDoseUnits.MilliroentgenEquivalentMan:
            return ((value * 100) / 0.001)
        
        return None


    def __convert_to_base(self, value: float, to_unit: RadiationEquivalentDoseUnits) -> float:
        
        if to_unit == RadiationEquivalentDoseUnits.Sievert:
            return (value)
        
        if to_unit == RadiationEquivalentDoseUnits.RoentgenEquivalentMan:
            return (value / 100)
        
        if to_unit == RadiationEquivalentDoseUnits.Nanosievert:
            return ((value) * 1e-09)
        
        if to_unit == RadiationEquivalentDoseUnits.Microsievert:
            return ((value) * 1e-06)
        
        if to_unit == RadiationEquivalentDoseUnits.Millisievert:
            return ((value) * 0.001)
        
        if to_unit == RadiationEquivalentDoseUnits.MilliroentgenEquivalentMan:
            return ((value / 100) * 0.001)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_sieverts(sieverts: float):
        """
        Create a new instance of RadiationEquivalentDose from a value in sieverts.

        The sievert is a unit in the International System of Units (SI) intended to represent the stochastic health risk of ionizing radiation, which is defined as the probability of causing radiation-induced cancer and genetic damage.

        :param meters: The RadiationEquivalentDose value in sieverts.
        :type sieverts: float
        :return: A new instance of RadiationEquivalentDose.
        :rtype: RadiationEquivalentDose
        """
        return RadiationEquivalentDose(sieverts, RadiationEquivalentDoseUnits.Sievert)

    
    @staticmethod
    def from_roentgens_equivalent_man(roentgens_equivalent_man: float):
        """
        Create a new instance of RadiationEquivalentDose from a value in roentgens_equivalent_man.

        

        :param meters: The RadiationEquivalentDose value in roentgens_equivalent_man.
        :type roentgens_equivalent_man: float
        :return: A new instance of RadiationEquivalentDose.
        :rtype: RadiationEquivalentDose
        """
        return RadiationEquivalentDose(roentgens_equivalent_man, RadiationEquivalentDoseUnits.RoentgenEquivalentMan)

    
    @staticmethod
    def from_nanosieverts(nanosieverts: float):
        """
        Create a new instance of RadiationEquivalentDose from a value in nanosieverts.

        

        :param meters: The RadiationEquivalentDose value in nanosieverts.
        :type nanosieverts: float
        :return: A new instance of RadiationEquivalentDose.
        :rtype: RadiationEquivalentDose
        """
        return RadiationEquivalentDose(nanosieverts, RadiationEquivalentDoseUnits.Nanosievert)

    
    @staticmethod
    def from_microsieverts(microsieverts: float):
        """
        Create a new instance of RadiationEquivalentDose from a value in microsieverts.

        

        :param meters: The RadiationEquivalentDose value in microsieverts.
        :type microsieverts: float
        :return: A new instance of RadiationEquivalentDose.
        :rtype: RadiationEquivalentDose
        """
        return RadiationEquivalentDose(microsieverts, RadiationEquivalentDoseUnits.Microsievert)

    
    @staticmethod
    def from_millisieverts(millisieverts: float):
        """
        Create a new instance of RadiationEquivalentDose from a value in millisieverts.

        

        :param meters: The RadiationEquivalentDose value in millisieverts.
        :type millisieverts: float
        :return: A new instance of RadiationEquivalentDose.
        :rtype: RadiationEquivalentDose
        """
        return RadiationEquivalentDose(millisieverts, RadiationEquivalentDoseUnits.Millisievert)

    
    @staticmethod
    def from_milliroentgens_equivalent_man(milliroentgens_equivalent_man: float):
        """
        Create a new instance of RadiationEquivalentDose from a value in milliroentgens_equivalent_man.

        

        :param meters: The RadiationEquivalentDose value in milliroentgens_equivalent_man.
        :type milliroentgens_equivalent_man: float
        :return: A new instance of RadiationEquivalentDose.
        :rtype: RadiationEquivalentDose
        """
        return RadiationEquivalentDose(milliroentgens_equivalent_man, RadiationEquivalentDoseUnits.MilliroentgenEquivalentMan)

    
    @property
    def sieverts(self) -> float:
        """
        The sievert is a unit in the International System of Units (SI) intended to represent the stochastic health risk of ionizing radiation, which is defined as the probability of causing radiation-induced cancer and genetic damage.
        """
        if self.__sieverts != None:
            return self.__sieverts
        self.__sieverts = self.__convert_from_base(RadiationEquivalentDoseUnits.Sievert)
        return self.__sieverts

    
    @property
    def roentgens_equivalent_man(self) -> float:
        """
        
        """
        if self.__roentgens_equivalent_man != None:
            return self.__roentgens_equivalent_man
        self.__roentgens_equivalent_man = self.__convert_from_base(RadiationEquivalentDoseUnits.RoentgenEquivalentMan)
        return self.__roentgens_equivalent_man

    
    @property
    def nanosieverts(self) -> float:
        """
        
        """
        if self.__nanosieverts != None:
            return self.__nanosieverts
        self.__nanosieverts = self.__convert_from_base(RadiationEquivalentDoseUnits.Nanosievert)
        return self.__nanosieverts

    
    @property
    def microsieverts(self) -> float:
        """
        
        """
        if self.__microsieverts != None:
            return self.__microsieverts
        self.__microsieverts = self.__convert_from_base(RadiationEquivalentDoseUnits.Microsievert)
        return self.__microsieverts

    
    @property
    def millisieverts(self) -> float:
        """
        
        """
        if self.__millisieverts != None:
            return self.__millisieverts
        self.__millisieverts = self.__convert_from_base(RadiationEquivalentDoseUnits.Millisievert)
        return self.__millisieverts

    
    @property
    def milliroentgens_equivalent_man(self) -> float:
        """
        
        """
        if self.__milliroentgens_equivalent_man != None:
            return self.__milliroentgens_equivalent_man
        self.__milliroentgens_equivalent_man = self.__convert_from_base(RadiationEquivalentDoseUnits.MilliroentgenEquivalentMan)
        return self.__milliroentgens_equivalent_man

    
    def to_string(self, unit: RadiationEquivalentDoseUnits = RadiationEquivalentDoseUnits.Sievert, fractional_digits: int = None) -> str:
        """
        Format the RadiationEquivalentDose to a string.
        
        Note: the default format for RadiationEquivalentDose is Sievert.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the RadiationEquivalentDose. Default is 'Sievert'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == RadiationEquivalentDoseUnits.Sievert:
            return f"""{super()._truncate_fraction_digits(self.sieverts, fractional_digits)} Sv"""
        
        if unit == RadiationEquivalentDoseUnits.RoentgenEquivalentMan:
            return f"""{super()._truncate_fraction_digits(self.roentgens_equivalent_man, fractional_digits)} rem"""
        
        if unit == RadiationEquivalentDoseUnits.Nanosievert:
            return f"""{super()._truncate_fraction_digits(self.nanosieverts, fractional_digits)} nSv"""
        
        if unit == RadiationEquivalentDoseUnits.Microsievert:
            return f"""{super()._truncate_fraction_digits(self.microsieverts, fractional_digits)} μSv"""
        
        if unit == RadiationEquivalentDoseUnits.Millisievert:
            return f"""{super()._truncate_fraction_digits(self.millisieverts, fractional_digits)} mSv"""
        
        if unit == RadiationEquivalentDoseUnits.MilliroentgenEquivalentMan:
            return f"""{super()._truncate_fraction_digits(self.milliroentgens_equivalent_man, fractional_digits)} mrem"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: RadiationEquivalentDoseUnits = RadiationEquivalentDoseUnits.Sievert) -> str:
        """
        Get RadiationEquivalentDose unit abbreviation.
        Note! the default abbreviation for RadiationEquivalentDose is Sievert.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == RadiationEquivalentDoseUnits.Sievert:
            return """Sv"""
        
        if unit_abbreviation == RadiationEquivalentDoseUnits.RoentgenEquivalentMan:
            return """rem"""
        
        if unit_abbreviation == RadiationEquivalentDoseUnits.Nanosievert:
            return """nSv"""
        
        if unit_abbreviation == RadiationEquivalentDoseUnits.Microsievert:
            return """μSv"""
        
        if unit_abbreviation == RadiationEquivalentDoseUnits.Millisievert:
            return """mSv"""
        
        if unit_abbreviation == RadiationEquivalentDoseUnits.MilliroentgenEquivalentMan:
            return """mrem"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class RotationalStiffnessPerLengthUnits(Enum):
        """
            RotationalStiffnessPerLengthUnits enumeration
        """
        
        NewtonMeterPerRadianPerMeter = 'NewtonMeterPerRadianPerMeter'
        """
            
        """
        
        PoundForceFootPerDegreesPerFoot = 'PoundForceFootPerDegreesPerFoot'
        """
            
        """
        
        KilopoundForceFootPerDegreesPerFoot = 'KilopoundForceFootPerDegreesPerFoot'
        """
            
        """
        
        KilonewtonMeterPerRadianPerMeter = 'KilonewtonMeterPerRadianPerMeter'
        """
            
        """
        
        MeganewtonMeterPerRadianPerMeter = 'MeganewtonMeterPerRadianPerMeter'
        """
            
        """
        

class RotationalStiffnessPerLengthDto:
    """
    A DTO representation of a RotationalStiffnessPerLength

    Attributes:
        value (float): The value of the RotationalStiffnessPerLength.
        unit (RotationalStiffnessPerLengthUnits): The specific unit that the RotationalStiffnessPerLength value is representing.
    """

    def __init__(self, value: float, unit: RotationalStiffnessPerLengthUnits):
        """
        Create a new DTO representation of a RotationalStiffnessPerLength

        Parameters:
            value (float): The value of the RotationalStiffnessPerLength.
            unit (RotationalStiffnessPerLengthUnits): The specific unit that the RotationalStiffnessPerLength value is representing.
        """
        self.value: float = value
        """
        The value of the RotationalStiffnessPerLength
        """
        self.unit: RotationalStiffnessPerLengthUnits = unit
        """
        The specific unit that the RotationalStiffnessPerLength value is representing
        """

    def to_json(self):
        """
        Get a RotationalStiffnessPerLength DTO JSON object representing the current unit.

        :return: JSON object represents RotationalStiffnessPerLength DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "NewtonMeterPerRadianPerMeter"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of RotationalStiffnessPerLength DTO from a json representation.

        :param data: The RotationalStiffnessPerLength DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "NewtonMeterPerRadianPerMeter"}
        :return: A new instance of RotationalStiffnessPerLengthDto.
        :rtype: RotationalStiffnessPerLengthDto
        """
        return RotationalStiffnessPerLengthDto(value=data["value"], unit=RotationalStiffnessPerLengthUnits(data["unit"]))


class RotationalStiffnessPerLength(AbstractMeasure):
    """
    https://en.wikipedia.org/wiki/Stiffness#Rotational_stiffness

    Args:
        value (float): The value.
        from_unit (RotationalStiffnessPerLengthUnits): The RotationalStiffnessPerLength unit to create from, The default unit is NewtonMeterPerRadianPerMeter
    """
    def __init__(self, value: float, from_unit: RotationalStiffnessPerLengthUnits = RotationalStiffnessPerLengthUnits.NewtonMeterPerRadianPerMeter):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__newton_meters_per_radian_per_meter = None
        
        self.__pound_force_feet_per_degrees_per_feet = None
        
        self.__kilopound_force_feet_per_degrees_per_feet = None
        
        self.__kilonewton_meters_per_radian_per_meter = None
        
        self.__meganewton_meters_per_radian_per_meter = None
        

    def convert(self, unit: RotationalStiffnessPerLengthUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: RotationalStiffnessPerLengthUnits = RotationalStiffnessPerLengthUnits.NewtonMeterPerRadianPerMeter) -> RotationalStiffnessPerLengthDto:
        """
        Get a new instance of RotationalStiffnessPerLength DTO representing the current unit.

        :param hold_in_unit: The specific RotationalStiffnessPerLength unit to store the RotationalStiffnessPerLength value in the DTO representation.
        :type hold_in_unit: RotationalStiffnessPerLengthUnits
        :return: A new instance of RotationalStiffnessPerLengthDto.
        :rtype: RotationalStiffnessPerLengthDto
        """
        return RotationalStiffnessPerLengthDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: RotationalStiffnessPerLengthUnits = RotationalStiffnessPerLengthUnits.NewtonMeterPerRadianPerMeter):
        """
        Get a RotationalStiffnessPerLength DTO JSON object representing the current unit.

        :param hold_in_unit: The specific RotationalStiffnessPerLength unit to store the RotationalStiffnessPerLength value in the DTO representation.
        :type hold_in_unit: RotationalStiffnessPerLengthUnits
        :return: JSON object represents RotationalStiffnessPerLength DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "NewtonMeterPerRadianPerMeter"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(rotational_stiffness_per_length_dto: RotationalStiffnessPerLengthDto):
        """
        Obtain a new instance of RotationalStiffnessPerLength from a DTO unit object.

        :param rotational_stiffness_per_length_dto: The RotationalStiffnessPerLength DTO representation.
        :type rotational_stiffness_per_length_dto: RotationalStiffnessPerLengthDto
        :return: A new instance of RotationalStiffnessPerLength.
        :rtype: RotationalStiffnessPerLength
        """
        return RotationalStiffnessPerLength(rotational_stiffness_per_length_dto.value, rotational_stiffness_per_length_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of RotationalStiffnessPerLength from a DTO unit json representation.

        :param data: The RotationalStiffnessPerLength DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "NewtonMeterPerRadianPerMeter"}
        :return: A new instance of RotationalStiffnessPerLength.
        :rtype: RotationalStiffnessPerLength
        """
        return RotationalStiffnessPerLength.from_dto(RotationalStiffnessPerLengthDto.from_json(data))

    def __convert_from_base(self, from_unit: RotationalStiffnessPerLengthUnits) -> float:
        value = self._value
        
        if from_unit == RotationalStiffnessPerLengthUnits.NewtonMeterPerRadianPerMeter:
            return (value)
        
        if from_unit == RotationalStiffnessPerLengthUnits.PoundForceFootPerDegreesPerFoot:
            return (value / 254.864324570)
        
        if from_unit == RotationalStiffnessPerLengthUnits.KilopoundForceFootPerDegreesPerFoot:
            return (value / 254864.324570)
        
        if from_unit == RotationalStiffnessPerLengthUnits.KilonewtonMeterPerRadianPerMeter:
            return ((value) / 1000.0)
        
        if from_unit == RotationalStiffnessPerLengthUnits.MeganewtonMeterPerRadianPerMeter:
            return ((value) / 1000000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: RotationalStiffnessPerLengthUnits) -> float:
        
        if to_unit == RotationalStiffnessPerLengthUnits.NewtonMeterPerRadianPerMeter:
            return (value)
        
        if to_unit == RotationalStiffnessPerLengthUnits.PoundForceFootPerDegreesPerFoot:
            return (value * 254.864324570)
        
        if to_unit == RotationalStiffnessPerLengthUnits.KilopoundForceFootPerDegreesPerFoot:
            return (value * 254864.324570)
        
        if to_unit == RotationalStiffnessPerLengthUnits.KilonewtonMeterPerRadianPerMeter:
            return ((value) * 1000.0)
        
        if to_unit == RotationalStiffnessPerLengthUnits.MeganewtonMeterPerRadianPerMeter:
            return ((value) * 1000000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_newton_meters_per_radian_per_meter(newton_meters_per_radian_per_meter: float):
        """
        Create a new instance of RotationalStiffnessPerLength from a value in newton_meters_per_radian_per_meter.

        

        :param meters: The RotationalStiffnessPerLength value in newton_meters_per_radian_per_meter.
        :type newton_meters_per_radian_per_meter: float
        :return: A new instance of RotationalStiffnessPerLength.
        :rtype: RotationalStiffnessPerLength
        """
        return RotationalStiffnessPerLength(newton_meters_per_radian_per_meter, RotationalStiffnessPerLengthUnits.NewtonMeterPerRadianPerMeter)

    
    @staticmethod
    def from_pound_force_feet_per_degrees_per_feet(pound_force_feet_per_degrees_per_feet: float):
        """
        Create a new instance of RotationalStiffnessPerLength from a value in pound_force_feet_per_degrees_per_feet.

        

        :param meters: The RotationalStiffnessPerLength value in pound_force_feet_per_degrees_per_feet.
        :type pound_force_feet_per_degrees_per_feet: float
        :return: A new instance of RotationalStiffnessPerLength.
        :rtype: RotationalStiffnessPerLength
        """
        return RotationalStiffnessPerLength(pound_force_feet_per_degrees_per_feet, RotationalStiffnessPerLengthUnits.PoundForceFootPerDegreesPerFoot)

    
    @staticmethod
    def from_kilopound_force_feet_per_degrees_per_feet(kilopound_force_feet_per_degrees_per_feet: float):
        """
        Create a new instance of RotationalStiffnessPerLength from a value in kilopound_force_feet_per_degrees_per_feet.

        

        :param meters: The RotationalStiffnessPerLength value in kilopound_force_feet_per_degrees_per_feet.
        :type kilopound_force_feet_per_degrees_per_feet: float
        :return: A new instance of RotationalStiffnessPerLength.
        :rtype: RotationalStiffnessPerLength
        """
        return RotationalStiffnessPerLength(kilopound_force_feet_per_degrees_per_feet, RotationalStiffnessPerLengthUnits.KilopoundForceFootPerDegreesPerFoot)

    
    @staticmethod
    def from_kilonewton_meters_per_radian_per_meter(kilonewton_meters_per_radian_per_meter: float):
        """
        Create a new instance of RotationalStiffnessPerLength from a value in kilonewton_meters_per_radian_per_meter.

        

        :param meters: The RotationalStiffnessPerLength value in kilonewton_meters_per_radian_per_meter.
        :type kilonewton_meters_per_radian_per_meter: float
        :return: A new instance of RotationalStiffnessPerLength.
        :rtype: RotationalStiffnessPerLength
        """
        return RotationalStiffnessPerLength(kilonewton_meters_per_radian_per_meter, RotationalStiffnessPerLengthUnits.KilonewtonMeterPerRadianPerMeter)

    
    @staticmethod
    def from_meganewton_meters_per_radian_per_meter(meganewton_meters_per_radian_per_meter: float):
        """
        Create a new instance of RotationalStiffnessPerLength from a value in meganewton_meters_per_radian_per_meter.

        

        :param meters: The RotationalStiffnessPerLength value in meganewton_meters_per_radian_per_meter.
        :type meganewton_meters_per_radian_per_meter: float
        :return: A new instance of RotationalStiffnessPerLength.
        :rtype: RotationalStiffnessPerLength
        """
        return RotationalStiffnessPerLength(meganewton_meters_per_radian_per_meter, RotationalStiffnessPerLengthUnits.MeganewtonMeterPerRadianPerMeter)

    
    @property
    def newton_meters_per_radian_per_meter(self) -> float:
        """
        
        """
        if self.__newton_meters_per_radian_per_meter != None:
            return self.__newton_meters_per_radian_per_meter
        self.__newton_meters_per_radian_per_meter = self.__convert_from_base(RotationalStiffnessPerLengthUnits.NewtonMeterPerRadianPerMeter)
        return self.__newton_meters_per_radian_per_meter

    
    @property
    def pound_force_feet_per_degrees_per_feet(self) -> float:
        """
        
        """
        if self.__pound_force_feet_per_degrees_per_feet != None:
            return self.__pound_force_feet_per_degrees_per_feet
        self.__pound_force_feet_per_degrees_per_feet = self.__convert_from_base(RotationalStiffnessPerLengthUnits.PoundForceFootPerDegreesPerFoot)
        return self.__pound_force_feet_per_degrees_per_feet

    
    @property
    def kilopound_force_feet_per_degrees_per_feet(self) -> float:
        """
        
        """
        if self.__kilopound_force_feet_per_degrees_per_feet != None:
            return self.__kilopound_force_feet_per_degrees_per_feet
        self.__kilopound_force_feet_per_degrees_per_feet = self.__convert_from_base(RotationalStiffnessPerLengthUnits.KilopoundForceFootPerDegreesPerFoot)
        return self.__kilopound_force_feet_per_degrees_per_feet

    
    @property
    def kilonewton_meters_per_radian_per_meter(self) -> float:
        """
        
        """
        if self.__kilonewton_meters_per_radian_per_meter != None:
            return self.__kilonewton_meters_per_radian_per_meter
        self.__kilonewton_meters_per_radian_per_meter = self.__convert_from_base(RotationalStiffnessPerLengthUnits.KilonewtonMeterPerRadianPerMeter)
        return self.__kilonewton_meters_per_radian_per_meter

    
    @property
    def meganewton_meters_per_radian_per_meter(self) -> float:
        """
        
        """
        if self.__meganewton_meters_per_radian_per_meter != None:
            return self.__meganewton_meters_per_radian_per_meter
        self.__meganewton_meters_per_radian_per_meter = self.__convert_from_base(RotationalStiffnessPerLengthUnits.MeganewtonMeterPerRadianPerMeter)
        return self.__meganewton_meters_per_radian_per_meter

    
    def to_string(self, unit: RotationalStiffnessPerLengthUnits = RotationalStiffnessPerLengthUnits.NewtonMeterPerRadianPerMeter, fractional_digits: int = None) -> str:
        """
        Format the RotationalStiffnessPerLength to a string.
        
        Note: the default format for RotationalStiffnessPerLength is NewtonMeterPerRadianPerMeter.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the RotationalStiffnessPerLength. Default is 'NewtonMeterPerRadianPerMeter'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == RotationalStiffnessPerLengthUnits.NewtonMeterPerRadianPerMeter:
            return f"""{super()._truncate_fraction_digits(self.newton_meters_per_radian_per_meter, fractional_digits)} N·m/rad/m"""
        
        if unit == RotationalStiffnessPerLengthUnits.PoundForceFootPerDegreesPerFoot:
            return f"""{super()._truncate_fraction_digits(self.pound_force_feet_per_degrees_per_feet, fractional_digits)} lbf·ft/deg/ft"""
        
        if unit == RotationalStiffnessPerLengthUnits.KilopoundForceFootPerDegreesPerFoot:
            return f"""{super()._truncate_fraction_digits(self.kilopound_force_feet_per_degrees_per_feet, fractional_digits)} kipf·ft/°/ft"""
        
        if unit == RotationalStiffnessPerLengthUnits.KilonewtonMeterPerRadianPerMeter:
            return f"""{super()._truncate_fraction_digits(self.kilonewton_meters_per_radian_per_meter, fractional_digits)} kN·m/rad/m"""
        
        if unit == RotationalStiffnessPerLengthUnits.MeganewtonMeterPerRadianPerMeter:
            return f"""{super()._truncate_fraction_digits(self.meganewton_meters_per_radian_per_meter, fractional_digits)} MN·m/rad/m"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: RotationalStiffnessPerLengthUnits = RotationalStiffnessPerLengthUnits.NewtonMeterPerRadianPerMeter) -> str:
        """
        Get RotationalStiffnessPerLength unit abbreviation.
        Note! the default abbreviation for RotationalStiffnessPerLength is NewtonMeterPerRadianPerMeter.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == RotationalStiffnessPerLengthUnits.NewtonMeterPerRadianPerMeter:
            return """N·m/rad/m"""
        
        if unit_abbreviation == RotationalStiffnessPerLengthUnits.PoundForceFootPerDegreesPerFoot:
            return """lbf·ft/deg/ft"""
        
        if unit_abbreviation == RotationalStiffnessPerLengthUnits.KilopoundForceFootPerDegreesPerFoot:
            return """kipf·ft/°/ft"""
        
        if unit_abbreviation == RotationalStiffnessPerLengthUnits.KilonewtonMeterPerRadianPerMeter:
            return """kN·m/rad/m"""
        
        if unit_abbreviation == RotationalStiffnessPerLengthUnits.MeganewtonMeterPerRadianPerMeter:
            return """MN·m/rad/m"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class LeakRateUnits(Enum):
        """
            LeakRateUnits enumeration
        """
        
        PascalCubicMeterPerSecond = 'PascalCubicMeterPerSecond'
        """
            
        """
        
        MillibarLiterPerSecond = 'MillibarLiterPerSecond'
        """
            
        """
        
        TorrLiterPerSecond = 'TorrLiterPerSecond'
        """
            
        """
        

class LeakRateDto:
    """
    A DTO representation of a LeakRate

    Attributes:
        value (float): The value of the LeakRate.
        unit (LeakRateUnits): The specific unit that the LeakRate value is representing.
    """

    def __init__(self, value: float, unit: LeakRateUnits):
        """
        Create a new DTO representation of a LeakRate

        Parameters:
            value (float): The value of the LeakRate.
            unit (LeakRateUnits): The specific unit that the LeakRate value is representing.
        """
        self.value: float = value
        """
        The value of the LeakRate
        """
        self.unit: LeakRateUnits = unit
        """
        The specific unit that the LeakRate value is representing
        """

    def to_json(self):
        """
        Get a LeakRate DTO JSON object representing the current unit.

        :return: JSON object represents LeakRate DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "PascalCubicMeterPerSecond"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of LeakRate DTO from a json representation.

        :param data: The LeakRate DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "PascalCubicMeterPerSecond"}
        :return: A new instance of LeakRateDto.
        :rtype: LeakRateDto
        """
        return LeakRateDto(value=data["value"], unit=LeakRateUnits(data["unit"]))


class LeakRate(AbstractMeasure):
    """
    A leakage rate of QL = 1 Pa-m³/s is given when the pressure in a closed, evacuated container with a volume of 1 m³ rises by 1 Pa per second or when the pressure in the container drops by 1 Pa in the event of overpressure.

    Args:
        value (float): The value.
        from_unit (LeakRateUnits): The LeakRate unit to create from, The default unit is PascalCubicMeterPerSecond
    """
    def __init__(self, value: float, from_unit: LeakRateUnits = LeakRateUnits.PascalCubicMeterPerSecond):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__pascal_cubic_meters_per_second = None
        
        self.__millibar_liters_per_second = None
        
        self.__torr_liters_per_second = None
        

    def convert(self, unit: LeakRateUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: LeakRateUnits = LeakRateUnits.PascalCubicMeterPerSecond) -> LeakRateDto:
        """
        Get a new instance of LeakRate DTO representing the current unit.

        :param hold_in_unit: The specific LeakRate unit to store the LeakRate value in the DTO representation.
        :type hold_in_unit: LeakRateUnits
        :return: A new instance of LeakRateDto.
        :rtype: LeakRateDto
        """
        return LeakRateDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: LeakRateUnits = LeakRateUnits.PascalCubicMeterPerSecond):
        """
        Get a LeakRate DTO JSON object representing the current unit.

        :param hold_in_unit: The specific LeakRate unit to store the LeakRate value in the DTO representation.
        :type hold_in_unit: LeakRateUnits
        :return: JSON object represents LeakRate DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "PascalCubicMeterPerSecond"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(leak_rate_dto: LeakRateDto):
        """
        Obtain a new instance of LeakRate from a DTO unit object.

        :param leak_rate_dto: The LeakRate DTO representation.
        :type leak_rate_dto: LeakRateDto
        :return: A new instance of LeakRate.
        :rtype: LeakRate
        """
        return LeakRate(leak_rate_dto.value, leak_rate_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of LeakRate from a DTO unit json representation.

        :param data: The LeakRate DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "PascalCubicMeterPerSecond"}
        :return: A new instance of LeakRate.
        :rtype: LeakRate
        """
        return LeakRate.from_dto(LeakRateDto.from_json(data))

    def __convert_from_base(self, from_unit: LeakRateUnits) -> float:
        value = self._value
        
        if from_unit == LeakRateUnits.PascalCubicMeterPerSecond:
            return (value)
        
        if from_unit == LeakRateUnits.MillibarLiterPerSecond:
            return (value * 10)
        
        if from_unit == LeakRateUnits.TorrLiterPerSecond:
            return (value * 7.5)
        
        return None


    def __convert_to_base(self, value: float, to_unit: LeakRateUnits) -> float:
        
        if to_unit == LeakRateUnits.PascalCubicMeterPerSecond:
            return (value)
        
        if to_unit == LeakRateUnits.MillibarLiterPerSecond:
            return (value / 10)
        
        if to_unit == LeakRateUnits.TorrLiterPerSecond:
            return (value / 7.5)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_pascal_cubic_meters_per_second(pascal_cubic_meters_per_second: float):
        """
        Create a new instance of LeakRate from a value in pascal_cubic_meters_per_second.

        

        :param meters: The LeakRate value in pascal_cubic_meters_per_second.
        :type pascal_cubic_meters_per_second: float
        :return: A new instance of LeakRate.
        :rtype: LeakRate
        """
        return LeakRate(pascal_cubic_meters_per_second, LeakRateUnits.PascalCubicMeterPerSecond)

    
    @staticmethod
    def from_millibar_liters_per_second(millibar_liters_per_second: float):
        """
        Create a new instance of LeakRate from a value in millibar_liters_per_second.

        

        :param meters: The LeakRate value in millibar_liters_per_second.
        :type millibar_liters_per_second: float
        :return: A new instance of LeakRate.
        :rtype: LeakRate
        """
        return LeakRate(millibar_liters_per_second, LeakRateUnits.MillibarLiterPerSecond)

    
    @staticmethod
    def from_torr_liters_per_second(torr_liters_per_second: float):
        """
        Create a new instance of LeakRate from a value in torr_liters_per_second.

        

        :param meters: The LeakRate value in torr_liters_per_second.
        :type torr_liters_per_second: float
        :return: A new instance of LeakRate.
        :rtype: LeakRate
        """
        return LeakRate(torr_liters_per_second, LeakRateUnits.TorrLiterPerSecond)

    
    @property
    def pascal_cubic_meters_per_second(self) -> float:
        """
        
        """
        if self.__pascal_cubic_meters_per_second != None:
            return self.__pascal_cubic_meters_per_second
        self.__pascal_cubic_meters_per_second = self.__convert_from_base(LeakRateUnits.PascalCubicMeterPerSecond)
        return self.__pascal_cubic_meters_per_second

    
    @property
    def millibar_liters_per_second(self) -> float:
        """
        
        """
        if self.__millibar_liters_per_second != None:
            return self.__millibar_liters_per_second
        self.__millibar_liters_per_second = self.__convert_from_base(LeakRateUnits.MillibarLiterPerSecond)
        return self.__millibar_liters_per_second

    
    @property
    def torr_liters_per_second(self) -> float:
        """
        
        """
        if self.__torr_liters_per_second != None:
            return self.__torr_liters_per_second
        self.__torr_liters_per_second = self.__convert_from_base(LeakRateUnits.TorrLiterPerSecond)
        return self.__torr_liters_per_second

    
    def to_string(self, unit: LeakRateUnits = LeakRateUnits.PascalCubicMeterPerSecond, fractional_digits: int = None) -> str:
        """
        Format the LeakRate to a string.
        
        Note: the default format for LeakRate is PascalCubicMeterPerSecond.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the LeakRate. Default is 'PascalCubicMeterPerSecond'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == LeakRateUnits.PascalCubicMeterPerSecond:
            return f"""{super()._truncate_fraction_digits(self.pascal_cubic_meters_per_second, fractional_digits)} Pa·m³/s"""
        
        if unit == LeakRateUnits.MillibarLiterPerSecond:
            return f"""{super()._truncate_fraction_digits(self.millibar_liters_per_second, fractional_digits)} mbar·l/s"""
        
        if unit == LeakRateUnits.TorrLiterPerSecond:
            return f"""{super()._truncate_fraction_digits(self.torr_liters_per_second, fractional_digits)} Torr·l/s"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: LeakRateUnits = LeakRateUnits.PascalCubicMeterPerSecond) -> str:
        """
        Get LeakRate unit abbreviation.
        Note! the default abbreviation for LeakRate is PascalCubicMeterPerSecond.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == LeakRateUnits.PascalCubicMeterPerSecond:
            return """Pa·m³/s"""
        
        if unit_abbreviation == LeakRateUnits.MillibarLiterPerSecond:
            return """mbar·l/s"""
        
        if unit_abbreviation == LeakRateUnits.TorrLiterPerSecond:
            return """Torr·l/s"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class MolarEntropyUnits(Enum):
        """
            MolarEntropyUnits enumeration
        """
        
        JoulePerMoleKelvin = 'JoulePerMoleKelvin'
        """
            
        """
        
        KilojoulePerMoleKelvin = 'KilojoulePerMoleKelvin'
        """
            
        """
        
        MegajoulePerMoleKelvin = 'MegajoulePerMoleKelvin'
        """
            
        """
        

class MolarEntropyDto:
    """
    A DTO representation of a MolarEntropy

    Attributes:
        value (float): The value of the MolarEntropy.
        unit (MolarEntropyUnits): The specific unit that the MolarEntropy value is representing.
    """

    def __init__(self, value: float, unit: MolarEntropyUnits):
        """
        Create a new DTO representation of a MolarEntropy

        Parameters:
            value (float): The value of the MolarEntropy.
            unit (MolarEntropyUnits): The specific unit that the MolarEntropy value is representing.
        """
        self.value: float = value
        """
        The value of the MolarEntropy
        """
        self.unit: MolarEntropyUnits = unit
        """
        The specific unit that the MolarEntropy value is representing
        """

    def to_json(self):
        """
        Get a MolarEntropy DTO JSON object representing the current unit.

        :return: JSON object represents MolarEntropy DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "JoulePerMoleKelvin"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of MolarEntropy DTO from a json representation.

        :param data: The MolarEntropy DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "JoulePerMoleKelvin"}
        :return: A new instance of MolarEntropyDto.
        :rtype: MolarEntropyDto
        """
        return MolarEntropyDto(value=data["value"], unit=MolarEntropyUnits(data["unit"]))


class MolarEntropy(AbstractMeasure):
    """
    Molar entropy is amount of energy required to increase temperature of 1 mole substance by 1 Kelvin.

    Args:
        value (float): The value.
        from_unit (MolarEntropyUnits): The MolarEntropy unit to create from, The default unit is JoulePerMoleKelvin
    """
    def __init__(self, value: float, from_unit: MolarEntropyUnits = MolarEntropyUnits.JoulePerMoleKelvin):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__joules_per_mole_kelvin = None
        
        self.__kilojoules_per_mole_kelvin = None
        
        self.__megajoules_per_mole_kelvin = None
        

    def convert(self, unit: MolarEntropyUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: MolarEntropyUnits = MolarEntropyUnits.JoulePerMoleKelvin) -> MolarEntropyDto:
        """
        Get a new instance of MolarEntropy DTO representing the current unit.

        :param hold_in_unit: The specific MolarEntropy unit to store the MolarEntropy value in the DTO representation.
        :type hold_in_unit: MolarEntropyUnits
        :return: A new instance of MolarEntropyDto.
        :rtype: MolarEntropyDto
        """
        return MolarEntropyDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: MolarEntropyUnits = MolarEntropyUnits.JoulePerMoleKelvin):
        """
        Get a MolarEntropy DTO JSON object representing the current unit.

        :param hold_in_unit: The specific MolarEntropy unit to store the MolarEntropy value in the DTO representation.
        :type hold_in_unit: MolarEntropyUnits
        :return: JSON object represents MolarEntropy DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "JoulePerMoleKelvin"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(molar_entropy_dto: MolarEntropyDto):
        """
        Obtain a new instance of MolarEntropy from a DTO unit object.

        :param molar_entropy_dto: The MolarEntropy DTO representation.
        :type molar_entropy_dto: MolarEntropyDto
        :return: A new instance of MolarEntropy.
        :rtype: MolarEntropy
        """
        return MolarEntropy(molar_entropy_dto.value, molar_entropy_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of MolarEntropy from a DTO unit json representation.

        :param data: The MolarEntropy DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "JoulePerMoleKelvin"}
        :return: A new instance of MolarEntropy.
        :rtype: MolarEntropy
        """
        return MolarEntropy.from_dto(MolarEntropyDto.from_json(data))

    def __convert_from_base(self, from_unit: MolarEntropyUnits) -> float:
        value = self._value
        
        if from_unit == MolarEntropyUnits.JoulePerMoleKelvin:
            return (value)
        
        if from_unit == MolarEntropyUnits.KilojoulePerMoleKelvin:
            return ((value) / 1000.0)
        
        if from_unit == MolarEntropyUnits.MegajoulePerMoleKelvin:
            return ((value) / 1000000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: MolarEntropyUnits) -> float:
        
        if to_unit == MolarEntropyUnits.JoulePerMoleKelvin:
            return (value)
        
        if to_unit == MolarEntropyUnits.KilojoulePerMoleKelvin:
            return ((value) * 1000.0)
        
        if to_unit == MolarEntropyUnits.MegajoulePerMoleKelvin:
            return ((value) * 1000000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_joules_per_mole_kelvin(joules_per_mole_kelvin: float):
        """
        Create a new instance of MolarEntropy from a value in joules_per_mole_kelvin.

        

        :param meters: The MolarEntropy value in joules_per_mole_kelvin.
        :type joules_per_mole_kelvin: float
        :return: A new instance of MolarEntropy.
        :rtype: MolarEntropy
        """
        return MolarEntropy(joules_per_mole_kelvin, MolarEntropyUnits.JoulePerMoleKelvin)

    
    @staticmethod
    def from_kilojoules_per_mole_kelvin(kilojoules_per_mole_kelvin: float):
        """
        Create a new instance of MolarEntropy from a value in kilojoules_per_mole_kelvin.

        

        :param meters: The MolarEntropy value in kilojoules_per_mole_kelvin.
        :type kilojoules_per_mole_kelvin: float
        :return: A new instance of MolarEntropy.
        :rtype: MolarEntropy
        """
        return MolarEntropy(kilojoules_per_mole_kelvin, MolarEntropyUnits.KilojoulePerMoleKelvin)

    
    @staticmethod
    def from_megajoules_per_mole_kelvin(megajoules_per_mole_kelvin: float):
        """
        Create a new instance of MolarEntropy from a value in megajoules_per_mole_kelvin.

        

        :param meters: The MolarEntropy value in megajoules_per_mole_kelvin.
        :type megajoules_per_mole_kelvin: float
        :return: A new instance of MolarEntropy.
        :rtype: MolarEntropy
        """
        return MolarEntropy(megajoules_per_mole_kelvin, MolarEntropyUnits.MegajoulePerMoleKelvin)

    
    @property
    def joules_per_mole_kelvin(self) -> float:
        """
        
        """
        if self.__joules_per_mole_kelvin != None:
            return self.__joules_per_mole_kelvin
        self.__joules_per_mole_kelvin = self.__convert_from_base(MolarEntropyUnits.JoulePerMoleKelvin)
        return self.__joules_per_mole_kelvin

    
    @property
    def kilojoules_per_mole_kelvin(self) -> float:
        """
        
        """
        if self.__kilojoules_per_mole_kelvin != None:
            return self.__kilojoules_per_mole_kelvin
        self.__kilojoules_per_mole_kelvin = self.__convert_from_base(MolarEntropyUnits.KilojoulePerMoleKelvin)
        return self.__kilojoules_per_mole_kelvin

    
    @property
    def megajoules_per_mole_kelvin(self) -> float:
        """
        
        """
        if self.__megajoules_per_mole_kelvin != None:
            return self.__megajoules_per_mole_kelvin
        self.__megajoules_per_mole_kelvin = self.__convert_from_base(MolarEntropyUnits.MegajoulePerMoleKelvin)
        return self.__megajoules_per_mole_kelvin

    
    def to_string(self, unit: MolarEntropyUnits = MolarEntropyUnits.JoulePerMoleKelvin, fractional_digits: int = None) -> str:
        """
        Format the MolarEntropy to a string.
        
        Note: the default format for MolarEntropy is JoulePerMoleKelvin.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the MolarEntropy. Default is 'JoulePerMoleKelvin'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == MolarEntropyUnits.JoulePerMoleKelvin:
            return f"""{super()._truncate_fraction_digits(self.joules_per_mole_kelvin, fractional_digits)} J/(mol*K)"""
        
        if unit == MolarEntropyUnits.KilojoulePerMoleKelvin:
            return f"""{super()._truncate_fraction_digits(self.kilojoules_per_mole_kelvin, fractional_digits)} kJ/(mol*K)"""
        
        if unit == MolarEntropyUnits.MegajoulePerMoleKelvin:
            return f"""{super()._truncate_fraction_digits(self.megajoules_per_mole_kelvin, fractional_digits)} MJ/(mol*K)"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: MolarEntropyUnits = MolarEntropyUnits.JoulePerMoleKelvin) -> str:
        """
        Get MolarEntropy unit abbreviation.
        Note! the default abbreviation for MolarEntropy is JoulePerMoleKelvin.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == MolarEntropyUnits.JoulePerMoleKelvin:
            return """J/(mol*K)"""
        
        if unit_abbreviation == MolarEntropyUnits.KilojoulePerMoleKelvin:
            return """kJ/(mol*K)"""
        
        if unit_abbreviation == MolarEntropyUnits.MegajoulePerMoleKelvin:
            return """MJ/(mol*K)"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class MolarityUnits(Enum):
        """
            MolarityUnits enumeration
        """
        
        MolePerCubicMeter = 'MolePerCubicMeter'
        """
            
        """
        
        MolePerLiter = 'MolePerLiter'
        """
            
        """
        
        PoundMolePerCubicFoot = 'PoundMolePerCubicFoot'
        """
            
        """
        
        KilomolePerCubicMeter = 'KilomolePerCubicMeter'
        """
            
        """
        
        FemtomolePerLiter = 'FemtomolePerLiter'
        """
            
        """
        
        PicomolePerLiter = 'PicomolePerLiter'
        """
            
        """
        
        NanomolePerLiter = 'NanomolePerLiter'
        """
            
        """
        
        MicromolePerLiter = 'MicromolePerLiter'
        """
            
        """
        
        MillimolePerLiter = 'MillimolePerLiter'
        """
            
        """
        
        CentimolePerLiter = 'CentimolePerLiter'
        """
            
        """
        
        DecimolePerLiter = 'DecimolePerLiter'
        """
            
        """
        

class MolarityDto:
    """
    A DTO representation of a Molarity

    Attributes:
        value (float): The value of the Molarity.
        unit (MolarityUnits): The specific unit that the Molarity value is representing.
    """

    def __init__(self, value: float, unit: MolarityUnits):
        """
        Create a new DTO representation of a Molarity

        Parameters:
            value (float): The value of the Molarity.
            unit (MolarityUnits): The specific unit that the Molarity value is representing.
        """
        self.value: float = value
        """
        The value of the Molarity
        """
        self.unit: MolarityUnits = unit
        """
        The specific unit that the Molarity value is representing
        """

    def to_json(self):
        """
        Get a Molarity DTO JSON object representing the current unit.

        :return: JSON object represents Molarity DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "MolePerCubicMeter"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of Molarity DTO from a json representation.

        :param data: The Molarity DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "MolePerCubicMeter"}
        :return: A new instance of MolarityDto.
        :rtype: MolarityDto
        """
        return MolarityDto(value=data["value"], unit=MolarityUnits(data["unit"]))


class Molarity(AbstractMeasure):
    """
    Molar concentration, also called molarity, amount concentration or substance concentration, is a measure of the concentration of a solute in a solution, or of any chemical species, in terms of amount of substance in a given volume.

    Args:
        value (float): The value.
        from_unit (MolarityUnits): The Molarity unit to create from, The default unit is MolePerCubicMeter
    """
    def __init__(self, value: float, from_unit: MolarityUnits = MolarityUnits.MolePerCubicMeter):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__moles_per_cubic_meter = None
        
        self.__moles_per_liter = None
        
        self.__pound_moles_per_cubic_foot = None
        
        self.__kilomoles_per_cubic_meter = None
        
        self.__femtomoles_per_liter = None
        
        self.__picomoles_per_liter = None
        
        self.__nanomoles_per_liter = None
        
        self.__micromoles_per_liter = None
        
        self.__millimoles_per_liter = None
        
        self.__centimoles_per_liter = None
        
        self.__decimoles_per_liter = None
        

    def convert(self, unit: MolarityUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: MolarityUnits = MolarityUnits.MolePerCubicMeter) -> MolarityDto:
        """
        Get a new instance of Molarity DTO representing the current unit.

        :param hold_in_unit: The specific Molarity unit to store the Molarity value in the DTO representation.
        :type hold_in_unit: MolarityUnits
        :return: A new instance of MolarityDto.
        :rtype: MolarityDto
        """
        return MolarityDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: MolarityUnits = MolarityUnits.MolePerCubicMeter):
        """
        Get a Molarity DTO JSON object representing the current unit.

        :param hold_in_unit: The specific Molarity unit to store the Molarity value in the DTO representation.
        :type hold_in_unit: MolarityUnits
        :return: JSON object represents Molarity DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "MolePerCubicMeter"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(molarity_dto: MolarityDto):
        """
        Obtain a new instance of Molarity from a DTO unit object.

        :param molarity_dto: The Molarity DTO representation.
        :type molarity_dto: MolarityDto
        :return: A new instance of Molarity.
        :rtype: Molarity
        """
        return Molarity(molarity_dto.value, molarity_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of Molarity from a DTO unit json representation.

        :param data: The Molarity DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "MolePerCubicMeter"}
        :return: A new instance of Molarity.
        :rtype: Molarity
        """
        return Molarity.from_dto(MolarityDto.from_json(data))

    def __convert_from_base(self, from_unit: MolarityUnits) -> float:
        value = self._value
        
        if from_unit == MolarityUnits.MolePerCubicMeter:
            return (value)
        
        if from_unit == MolarityUnits.MolePerLiter:
            return (value * 1e-3)
        
        if from_unit == MolarityUnits.PoundMolePerCubicFoot:
            return (value * 6.2427960576144611956325455827221e-5)
        
        if from_unit == MolarityUnits.KilomolePerCubicMeter:
            return ((value) / 1000.0)
        
        if from_unit == MolarityUnits.FemtomolePerLiter:
            return ((value * 1e-3) / 1e-15)
        
        if from_unit == MolarityUnits.PicomolePerLiter:
            return ((value * 1e-3) / 1e-12)
        
        if from_unit == MolarityUnits.NanomolePerLiter:
            return ((value * 1e-3) / 1e-09)
        
        if from_unit == MolarityUnits.MicromolePerLiter:
            return ((value * 1e-3) / 1e-06)
        
        if from_unit == MolarityUnits.MillimolePerLiter:
            return ((value * 1e-3) / 0.001)
        
        if from_unit == MolarityUnits.CentimolePerLiter:
            return ((value * 1e-3) / 0.01)
        
        if from_unit == MolarityUnits.DecimolePerLiter:
            return ((value * 1e-3) / 0.1)
        
        return None


    def __convert_to_base(self, value: float, to_unit: MolarityUnits) -> float:
        
        if to_unit == MolarityUnits.MolePerCubicMeter:
            return (value)
        
        if to_unit == MolarityUnits.MolePerLiter:
            return (value / 1e-3)
        
        if to_unit == MolarityUnits.PoundMolePerCubicFoot:
            return (value / 6.2427960576144611956325455827221e-5)
        
        if to_unit == MolarityUnits.KilomolePerCubicMeter:
            return ((value) * 1000.0)
        
        if to_unit == MolarityUnits.FemtomolePerLiter:
            return ((value / 1e-3) * 1e-15)
        
        if to_unit == MolarityUnits.PicomolePerLiter:
            return ((value / 1e-3) * 1e-12)
        
        if to_unit == MolarityUnits.NanomolePerLiter:
            return ((value / 1e-3) * 1e-09)
        
        if to_unit == MolarityUnits.MicromolePerLiter:
            return ((value / 1e-3) * 1e-06)
        
        if to_unit == MolarityUnits.MillimolePerLiter:
            return ((value / 1e-3) * 0.001)
        
        if to_unit == MolarityUnits.CentimolePerLiter:
            return ((value / 1e-3) * 0.01)
        
        if to_unit == MolarityUnits.DecimolePerLiter:
            return ((value / 1e-3) * 0.1)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_moles_per_cubic_meter(moles_per_cubic_meter: float):
        """
        Create a new instance of Molarity from a value in moles_per_cubic_meter.

        

        :param meters: The Molarity value in moles_per_cubic_meter.
        :type moles_per_cubic_meter: float
        :return: A new instance of Molarity.
        :rtype: Molarity
        """
        return Molarity(moles_per_cubic_meter, MolarityUnits.MolePerCubicMeter)

    
    @staticmethod
    def from_moles_per_liter(moles_per_liter: float):
        """
        Create a new instance of Molarity from a value in moles_per_liter.

        

        :param meters: The Molarity value in moles_per_liter.
        :type moles_per_liter: float
        :return: A new instance of Molarity.
        :rtype: Molarity
        """
        return Molarity(moles_per_liter, MolarityUnits.MolePerLiter)

    
    @staticmethod
    def from_pound_moles_per_cubic_foot(pound_moles_per_cubic_foot: float):
        """
        Create a new instance of Molarity from a value in pound_moles_per_cubic_foot.

        

        :param meters: The Molarity value in pound_moles_per_cubic_foot.
        :type pound_moles_per_cubic_foot: float
        :return: A new instance of Molarity.
        :rtype: Molarity
        """
        return Molarity(pound_moles_per_cubic_foot, MolarityUnits.PoundMolePerCubicFoot)

    
    @staticmethod
    def from_kilomoles_per_cubic_meter(kilomoles_per_cubic_meter: float):
        """
        Create a new instance of Molarity from a value in kilomoles_per_cubic_meter.

        

        :param meters: The Molarity value in kilomoles_per_cubic_meter.
        :type kilomoles_per_cubic_meter: float
        :return: A new instance of Molarity.
        :rtype: Molarity
        """
        return Molarity(kilomoles_per_cubic_meter, MolarityUnits.KilomolePerCubicMeter)

    
    @staticmethod
    def from_femtomoles_per_liter(femtomoles_per_liter: float):
        """
        Create a new instance of Molarity from a value in femtomoles_per_liter.

        

        :param meters: The Molarity value in femtomoles_per_liter.
        :type femtomoles_per_liter: float
        :return: A new instance of Molarity.
        :rtype: Molarity
        """
        return Molarity(femtomoles_per_liter, MolarityUnits.FemtomolePerLiter)

    
    @staticmethod
    def from_picomoles_per_liter(picomoles_per_liter: float):
        """
        Create a new instance of Molarity from a value in picomoles_per_liter.

        

        :param meters: The Molarity value in picomoles_per_liter.
        :type picomoles_per_liter: float
        :return: A new instance of Molarity.
        :rtype: Molarity
        """
        return Molarity(picomoles_per_liter, MolarityUnits.PicomolePerLiter)

    
    @staticmethod
    def from_nanomoles_per_liter(nanomoles_per_liter: float):
        """
        Create a new instance of Molarity from a value in nanomoles_per_liter.

        

        :param meters: The Molarity value in nanomoles_per_liter.
        :type nanomoles_per_liter: float
        :return: A new instance of Molarity.
        :rtype: Molarity
        """
        return Molarity(nanomoles_per_liter, MolarityUnits.NanomolePerLiter)

    
    @staticmethod
    def from_micromoles_per_liter(micromoles_per_liter: float):
        """
        Create a new instance of Molarity from a value in micromoles_per_liter.

        

        :param meters: The Molarity value in micromoles_per_liter.
        :type micromoles_per_liter: float
        :return: A new instance of Molarity.
        :rtype: Molarity
        """
        return Molarity(micromoles_per_liter, MolarityUnits.MicromolePerLiter)

    
    @staticmethod
    def from_millimoles_per_liter(millimoles_per_liter: float):
        """
        Create a new instance of Molarity from a value in millimoles_per_liter.

        

        :param meters: The Molarity value in millimoles_per_liter.
        :type millimoles_per_liter: float
        :return: A new instance of Molarity.
        :rtype: Molarity
        """
        return Molarity(millimoles_per_liter, MolarityUnits.MillimolePerLiter)

    
    @staticmethod
    def from_centimoles_per_liter(centimoles_per_liter: float):
        """
        Create a new instance of Molarity from a value in centimoles_per_liter.

        

        :param meters: The Molarity value in centimoles_per_liter.
        :type centimoles_per_liter: float
        :return: A new instance of Molarity.
        :rtype: Molarity
        """
        return Molarity(centimoles_per_liter, MolarityUnits.CentimolePerLiter)

    
    @staticmethod
    def from_decimoles_per_liter(decimoles_per_liter: float):
        """
        Create a new instance of Molarity from a value in decimoles_per_liter.

        

        :param meters: The Molarity value in decimoles_per_liter.
        :type decimoles_per_liter: float
        :return: A new instance of Molarity.
        :rtype: Molarity
        """
        return Molarity(decimoles_per_liter, MolarityUnits.DecimolePerLiter)

    
    @property
    def moles_per_cubic_meter(self) -> float:
        """
        
        """
        if self.__moles_per_cubic_meter != None:
            return self.__moles_per_cubic_meter
        self.__moles_per_cubic_meter = self.__convert_from_base(MolarityUnits.MolePerCubicMeter)
        return self.__moles_per_cubic_meter

    
    @property
    def moles_per_liter(self) -> float:
        """
        
        """
        if self.__moles_per_liter != None:
            return self.__moles_per_liter
        self.__moles_per_liter = self.__convert_from_base(MolarityUnits.MolePerLiter)
        return self.__moles_per_liter

    
    @property
    def pound_moles_per_cubic_foot(self) -> float:
        """
        
        """
        if self.__pound_moles_per_cubic_foot != None:
            return self.__pound_moles_per_cubic_foot
        self.__pound_moles_per_cubic_foot = self.__convert_from_base(MolarityUnits.PoundMolePerCubicFoot)
        return self.__pound_moles_per_cubic_foot

    
    @property
    def kilomoles_per_cubic_meter(self) -> float:
        """
        
        """
        if self.__kilomoles_per_cubic_meter != None:
            return self.__kilomoles_per_cubic_meter
        self.__kilomoles_per_cubic_meter = self.__convert_from_base(MolarityUnits.KilomolePerCubicMeter)
        return self.__kilomoles_per_cubic_meter

    
    @property
    def femtomoles_per_liter(self) -> float:
        """
        
        """
        if self.__femtomoles_per_liter != None:
            return self.__femtomoles_per_liter
        self.__femtomoles_per_liter = self.__convert_from_base(MolarityUnits.FemtomolePerLiter)
        return self.__femtomoles_per_liter

    
    @property
    def picomoles_per_liter(self) -> float:
        """
        
        """
        if self.__picomoles_per_liter != None:
            return self.__picomoles_per_liter
        self.__picomoles_per_liter = self.__convert_from_base(MolarityUnits.PicomolePerLiter)
        return self.__picomoles_per_liter

    
    @property
    def nanomoles_per_liter(self) -> float:
        """
        
        """
        if self.__nanomoles_per_liter != None:
            return self.__nanomoles_per_liter
        self.__nanomoles_per_liter = self.__convert_from_base(MolarityUnits.NanomolePerLiter)
        return self.__nanomoles_per_liter

    
    @property
    def micromoles_per_liter(self) -> float:
        """
        
        """
        if self.__micromoles_per_liter != None:
            return self.__micromoles_per_liter
        self.__micromoles_per_liter = self.__convert_from_base(MolarityUnits.MicromolePerLiter)
        return self.__micromoles_per_liter

    
    @property
    def millimoles_per_liter(self) -> float:
        """
        
        """
        if self.__millimoles_per_liter != None:
            return self.__millimoles_per_liter
        self.__millimoles_per_liter = self.__convert_from_base(MolarityUnits.MillimolePerLiter)
        return self.__millimoles_per_liter

    
    @property
    def centimoles_per_liter(self) -> float:
        """
        
        """
        if self.__centimoles_per_liter != None:
            return self.__centimoles_per_liter
        self.__centimoles_per_liter = self.__convert_from_base(MolarityUnits.CentimolePerLiter)
        return self.__centimoles_per_liter

    
    @property
    def decimoles_per_liter(self) -> float:
        """
        
        """
        if self.__decimoles_per_liter != None:
            return self.__decimoles_per_liter
        self.__decimoles_per_liter = self.__convert_from_base(MolarityUnits.DecimolePerLiter)
        return self.__decimoles_per_liter

    
    def to_string(self, unit: MolarityUnits = MolarityUnits.MolePerCubicMeter, fractional_digits: int = None) -> str:
        """
        Format the Molarity to a string.
        
        Note: the default format for Molarity is MolePerCubicMeter.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the Molarity. Default is 'MolePerCubicMeter'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == MolarityUnits.MolePerCubicMeter:
            return f"""{super()._truncate_fraction_digits(self.moles_per_cubic_meter, fractional_digits)} mol/m³"""
        
        if unit == MolarityUnits.MolePerLiter:
            return f"""{super()._truncate_fraction_digits(self.moles_per_liter, fractional_digits)} mol/L"""
        
        if unit == MolarityUnits.PoundMolePerCubicFoot:
            return f"""{super()._truncate_fraction_digits(self.pound_moles_per_cubic_foot, fractional_digits)} lbmol/ft³"""
        
        if unit == MolarityUnits.KilomolePerCubicMeter:
            return f"""{super()._truncate_fraction_digits(self.kilomoles_per_cubic_meter, fractional_digits)} kmol/m³"""
        
        if unit == MolarityUnits.FemtomolePerLiter:
            return f"""{super()._truncate_fraction_digits(self.femtomoles_per_liter, fractional_digits)} fmol/L"""
        
        if unit == MolarityUnits.PicomolePerLiter:
            return f"""{super()._truncate_fraction_digits(self.picomoles_per_liter, fractional_digits)} pmol/L"""
        
        if unit == MolarityUnits.NanomolePerLiter:
            return f"""{super()._truncate_fraction_digits(self.nanomoles_per_liter, fractional_digits)} nmol/L"""
        
        if unit == MolarityUnits.MicromolePerLiter:
            return f"""{super()._truncate_fraction_digits(self.micromoles_per_liter, fractional_digits)} μmol/L"""
        
        if unit == MolarityUnits.MillimolePerLiter:
            return f"""{super()._truncate_fraction_digits(self.millimoles_per_liter, fractional_digits)} mmol/L"""
        
        if unit == MolarityUnits.CentimolePerLiter:
            return f"""{super()._truncate_fraction_digits(self.centimoles_per_liter, fractional_digits)} cmol/L"""
        
        if unit == MolarityUnits.DecimolePerLiter:
            return f"""{super()._truncate_fraction_digits(self.decimoles_per_liter, fractional_digits)} dmol/L"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: MolarityUnits = MolarityUnits.MolePerCubicMeter) -> str:
        """
        Get Molarity unit abbreviation.
        Note! the default abbreviation for Molarity is MolePerCubicMeter.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == MolarityUnits.MolePerCubicMeter:
            return """mol/m³"""
        
        if unit_abbreviation == MolarityUnits.MolePerLiter:
            return """mol/L"""
        
        if unit_abbreviation == MolarityUnits.PoundMolePerCubicFoot:
            return """lbmol/ft³"""
        
        if unit_abbreviation == MolarityUnits.KilomolePerCubicMeter:
            return """kmol/m³"""
        
        if unit_abbreviation == MolarityUnits.FemtomolePerLiter:
            return """fmol/L"""
        
        if unit_abbreviation == MolarityUnits.PicomolePerLiter:
            return """pmol/L"""
        
        if unit_abbreviation == MolarityUnits.NanomolePerLiter:
            return """nmol/L"""
        
        if unit_abbreviation == MolarityUnits.MicromolePerLiter:
            return """μmol/L"""
        
        if unit_abbreviation == MolarityUnits.MillimolePerLiter:
            return """mmol/L"""
        
        if unit_abbreviation == MolarityUnits.CentimolePerLiter:
            return """cmol/L"""
        
        if unit_abbreviation == MolarityUnits.DecimolePerLiter:
            return """dmol/L"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class MassFlowUnits(Enum):
        """
            MassFlowUnits enumeration
        """
        
        GramPerSecond = 'GramPerSecond'
        """
            
        """
        
        GramPerDay = 'GramPerDay'
        """
            
        """
        
        GramPerHour = 'GramPerHour'
        """
            
        """
        
        KilogramPerHour = 'KilogramPerHour'
        """
            
        """
        
        KilogramPerMinute = 'KilogramPerMinute'
        """
            
        """
        
        TonnePerHour = 'TonnePerHour'
        """
            
        """
        
        PoundPerDay = 'PoundPerDay'
        """
            
        """
        
        PoundPerHour = 'PoundPerHour'
        """
            
        """
        
        PoundPerMinute = 'PoundPerMinute'
        """
            
        """
        
        PoundPerSecond = 'PoundPerSecond'
        """
            
        """
        
        TonnePerDay = 'TonnePerDay'
        """
            
        """
        
        ShortTonPerHour = 'ShortTonPerHour'
        """
            
        """
        
        NanogramPerSecond = 'NanogramPerSecond'
        """
            
        """
        
        MicrogramPerSecond = 'MicrogramPerSecond'
        """
            
        """
        
        MilligramPerSecond = 'MilligramPerSecond'
        """
            
        """
        
        CentigramPerSecond = 'CentigramPerSecond'
        """
            
        """
        
        DecigramPerSecond = 'DecigramPerSecond'
        """
            
        """
        
        DecagramPerSecond = 'DecagramPerSecond'
        """
            
        """
        
        HectogramPerSecond = 'HectogramPerSecond'
        """
            
        """
        
        KilogramPerSecond = 'KilogramPerSecond'
        """
            
        """
        
        NanogramPerDay = 'NanogramPerDay'
        """
            
        """
        
        MicrogramPerDay = 'MicrogramPerDay'
        """
            
        """
        
        MilligramPerDay = 'MilligramPerDay'
        """
            
        """
        
        CentigramPerDay = 'CentigramPerDay'
        """
            
        """
        
        DecigramPerDay = 'DecigramPerDay'
        """
            
        """
        
        DecagramPerDay = 'DecagramPerDay'
        """
            
        """
        
        HectogramPerDay = 'HectogramPerDay'
        """
            
        """
        
        KilogramPerDay = 'KilogramPerDay'
        """
            
        """
        
        MegagramPerDay = 'MegagramPerDay'
        """
            
        """
        
        MegapoundPerDay = 'MegapoundPerDay'
        """
            
        """
        
        MegapoundPerHour = 'MegapoundPerHour'
        """
            
        """
        
        MegapoundPerMinute = 'MegapoundPerMinute'
        """
            
        """
        
        MegapoundPerSecond = 'MegapoundPerSecond'
        """
            
        """
        

class MassFlowDto:
    """
    A DTO representation of a MassFlow

    Attributes:
        value (float): The value of the MassFlow.
        unit (MassFlowUnits): The specific unit that the MassFlow value is representing.
    """

    def __init__(self, value: float, unit: MassFlowUnits):
        """
        Create a new DTO representation of a MassFlow

        Parameters:
            value (float): The value of the MassFlow.
            unit (MassFlowUnits): The specific unit that the MassFlow value is representing.
        """
        self.value: float = value
        """
        The value of the MassFlow
        """
        self.unit: MassFlowUnits = unit
        """
        The specific unit that the MassFlow value is representing
        """

    def to_json(self):
        """
        Get a MassFlow DTO JSON object representing the current unit.

        :return: JSON object represents MassFlow DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "GramPerSecond"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of MassFlow DTO from a json representation.

        :param data: The MassFlow DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "GramPerSecond"}
        :return: A new instance of MassFlowDto.
        :rtype: MassFlowDto
        """
        return MassFlowDto(value=data["value"], unit=MassFlowUnits(data["unit"]))


class MassFlow(AbstractMeasure):
    """
    Mass flow is the ratio of the mass change to the time during which the change occurred (value of mass changes per unit time).

    Args:
        value (float): The value.
        from_unit (MassFlowUnits): The MassFlow unit to create from, The default unit is GramPerSecond
    """
    def __init__(self, value: float, from_unit: MassFlowUnits = MassFlowUnits.GramPerSecond):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__grams_per_second = None
        
        self.__grams_per_day = None
        
        self.__grams_per_hour = None
        
        self.__kilograms_per_hour = None
        
        self.__kilograms_per_minute = None
        
        self.__tonnes_per_hour = None
        
        self.__pounds_per_day = None
        
        self.__pounds_per_hour = None
        
        self.__pounds_per_minute = None
        
        self.__pounds_per_second = None
        
        self.__tonnes_per_day = None
        
        self.__short_tons_per_hour = None
        
        self.__nanograms_per_second = None
        
        self.__micrograms_per_second = None
        
        self.__milligrams_per_second = None
        
        self.__centigrams_per_second = None
        
        self.__decigrams_per_second = None
        
        self.__decagrams_per_second = None
        
        self.__hectograms_per_second = None
        
        self.__kilograms_per_second = None
        
        self.__nanograms_per_day = None
        
        self.__micrograms_per_day = None
        
        self.__milligrams_per_day = None
        
        self.__centigrams_per_day = None
        
        self.__decigrams_per_day = None
        
        self.__decagrams_per_day = None
        
        self.__hectograms_per_day = None
        
        self.__kilograms_per_day = None
        
        self.__megagrams_per_day = None
        
        self.__megapounds_per_day = None
        
        self.__megapounds_per_hour = None
        
        self.__megapounds_per_minute = None
        
        self.__megapounds_per_second = None
        

    def convert(self, unit: MassFlowUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: MassFlowUnits = MassFlowUnits.GramPerSecond) -> MassFlowDto:
        """
        Get a new instance of MassFlow DTO representing the current unit.

        :param hold_in_unit: The specific MassFlow unit to store the MassFlow value in the DTO representation.
        :type hold_in_unit: MassFlowUnits
        :return: A new instance of MassFlowDto.
        :rtype: MassFlowDto
        """
        return MassFlowDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: MassFlowUnits = MassFlowUnits.GramPerSecond):
        """
        Get a MassFlow DTO JSON object representing the current unit.

        :param hold_in_unit: The specific MassFlow unit to store the MassFlow value in the DTO representation.
        :type hold_in_unit: MassFlowUnits
        :return: JSON object represents MassFlow DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "GramPerSecond"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(mass_flow_dto: MassFlowDto):
        """
        Obtain a new instance of MassFlow from a DTO unit object.

        :param mass_flow_dto: The MassFlow DTO representation.
        :type mass_flow_dto: MassFlowDto
        :return: A new instance of MassFlow.
        :rtype: MassFlow
        """
        return MassFlow(mass_flow_dto.value, mass_flow_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of MassFlow from a DTO unit json representation.

        :param data: The MassFlow DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "GramPerSecond"}
        :return: A new instance of MassFlow.
        :rtype: MassFlow
        """
        return MassFlow.from_dto(MassFlowDto.from_json(data))

    def __convert_from_base(self, from_unit: MassFlowUnits) -> float:
        value = self._value
        
        if from_unit == MassFlowUnits.GramPerSecond:
            return (value)
        
        if from_unit == MassFlowUnits.GramPerDay:
            return (value * 86400)
        
        if from_unit == MassFlowUnits.GramPerHour:
            return (value * 3600)
        
        if from_unit == MassFlowUnits.KilogramPerHour:
            return (value * 3.6)
        
        if from_unit == MassFlowUnits.KilogramPerMinute:
            return (value * 0.06)
        
        if from_unit == MassFlowUnits.TonnePerHour:
            return (value * 3.6 / 1000)
        
        if from_unit == MassFlowUnits.PoundPerDay:
            return (value * 190.47936)
        
        if from_unit == MassFlowUnits.PoundPerHour:
            return (value * 7.93664)
        
        if from_unit == MassFlowUnits.PoundPerMinute:
            return (value * 0.132277)
        
        if from_unit == MassFlowUnits.PoundPerSecond:
            return (value / 453.59237)
        
        if from_unit == MassFlowUnits.TonnePerDay:
            return (value * 0.0864000)
        
        if from_unit == MassFlowUnits.ShortTonPerHour:
            return (value / 251.9957611)
        
        if from_unit == MassFlowUnits.NanogramPerSecond:
            return ((value) / 1e-09)
        
        if from_unit == MassFlowUnits.MicrogramPerSecond:
            return ((value) / 1e-06)
        
        if from_unit == MassFlowUnits.MilligramPerSecond:
            return ((value) / 0.001)
        
        if from_unit == MassFlowUnits.CentigramPerSecond:
            return ((value) / 0.01)
        
        if from_unit == MassFlowUnits.DecigramPerSecond:
            return ((value) / 0.1)
        
        if from_unit == MassFlowUnits.DecagramPerSecond:
            return ((value) / 10.0)
        
        if from_unit == MassFlowUnits.HectogramPerSecond:
            return ((value) / 100.0)
        
        if from_unit == MassFlowUnits.KilogramPerSecond:
            return ((value) / 1000.0)
        
        if from_unit == MassFlowUnits.NanogramPerDay:
            return ((value * 86400) / 1e-09)
        
        if from_unit == MassFlowUnits.MicrogramPerDay:
            return ((value * 86400) / 1e-06)
        
        if from_unit == MassFlowUnits.MilligramPerDay:
            return ((value * 86400) / 0.001)
        
        if from_unit == MassFlowUnits.CentigramPerDay:
            return ((value * 86400) / 0.01)
        
        if from_unit == MassFlowUnits.DecigramPerDay:
            return ((value * 86400) / 0.1)
        
        if from_unit == MassFlowUnits.DecagramPerDay:
            return ((value * 86400) / 10.0)
        
        if from_unit == MassFlowUnits.HectogramPerDay:
            return ((value * 86400) / 100.0)
        
        if from_unit == MassFlowUnits.KilogramPerDay:
            return ((value * 86400) / 1000.0)
        
        if from_unit == MassFlowUnits.MegagramPerDay:
            return ((value * 86400) / 1000000.0)
        
        if from_unit == MassFlowUnits.MegapoundPerDay:
            return ((value * 190.47936) / 1000000.0)
        
        if from_unit == MassFlowUnits.MegapoundPerHour:
            return ((value * 7.93664) / 1000000.0)
        
        if from_unit == MassFlowUnits.MegapoundPerMinute:
            return ((value * 0.132277) / 1000000.0)
        
        if from_unit == MassFlowUnits.MegapoundPerSecond:
            return ((value / 453.59237) / 1000000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: MassFlowUnits) -> float:
        
        if to_unit == MassFlowUnits.GramPerSecond:
            return (value)
        
        if to_unit == MassFlowUnits.GramPerDay:
            return (value / 86400)
        
        if to_unit == MassFlowUnits.GramPerHour:
            return (value / 3600)
        
        if to_unit == MassFlowUnits.KilogramPerHour:
            return (value / 3.6)
        
        if to_unit == MassFlowUnits.KilogramPerMinute:
            return (value / 0.06)
        
        if to_unit == MassFlowUnits.TonnePerHour:
            return (1000 * value / 3.6)
        
        if to_unit == MassFlowUnits.PoundPerDay:
            return (value / 190.47936)
        
        if to_unit == MassFlowUnits.PoundPerHour:
            return (value / 7.93664)
        
        if to_unit == MassFlowUnits.PoundPerMinute:
            return (value / 0.132277)
        
        if to_unit == MassFlowUnits.PoundPerSecond:
            return (value * 453.59237)
        
        if to_unit == MassFlowUnits.TonnePerDay:
            return (value / 0.0864000)
        
        if to_unit == MassFlowUnits.ShortTonPerHour:
            return (value * 251.9957611)
        
        if to_unit == MassFlowUnits.NanogramPerSecond:
            return ((value) * 1e-09)
        
        if to_unit == MassFlowUnits.MicrogramPerSecond:
            return ((value) * 1e-06)
        
        if to_unit == MassFlowUnits.MilligramPerSecond:
            return ((value) * 0.001)
        
        if to_unit == MassFlowUnits.CentigramPerSecond:
            return ((value) * 0.01)
        
        if to_unit == MassFlowUnits.DecigramPerSecond:
            return ((value) * 0.1)
        
        if to_unit == MassFlowUnits.DecagramPerSecond:
            return ((value) * 10.0)
        
        if to_unit == MassFlowUnits.HectogramPerSecond:
            return ((value) * 100.0)
        
        if to_unit == MassFlowUnits.KilogramPerSecond:
            return ((value) * 1000.0)
        
        if to_unit == MassFlowUnits.NanogramPerDay:
            return ((value / 86400) * 1e-09)
        
        if to_unit == MassFlowUnits.MicrogramPerDay:
            return ((value / 86400) * 1e-06)
        
        if to_unit == MassFlowUnits.MilligramPerDay:
            return ((value / 86400) * 0.001)
        
        if to_unit == MassFlowUnits.CentigramPerDay:
            return ((value / 86400) * 0.01)
        
        if to_unit == MassFlowUnits.DecigramPerDay:
            return ((value / 86400) * 0.1)
        
        if to_unit == MassFlowUnits.DecagramPerDay:
            return ((value / 86400) * 10.0)
        
        if to_unit == MassFlowUnits.HectogramPerDay:
            return ((value / 86400) * 100.0)
        
        if to_unit == MassFlowUnits.KilogramPerDay:
            return ((value / 86400) * 1000.0)
        
        if to_unit == MassFlowUnits.MegagramPerDay:
            return ((value / 86400) * 1000000.0)
        
        if to_unit == MassFlowUnits.MegapoundPerDay:
            return ((value / 190.47936) * 1000000.0)
        
        if to_unit == MassFlowUnits.MegapoundPerHour:
            return ((value / 7.93664) * 1000000.0)
        
        if to_unit == MassFlowUnits.MegapoundPerMinute:
            return ((value / 0.132277) * 1000000.0)
        
        if to_unit == MassFlowUnits.MegapoundPerSecond:
            return ((value * 453.59237) * 1000000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_grams_per_second(grams_per_second: float):
        """
        Create a new instance of MassFlow from a value in grams_per_second.

        

        :param meters: The MassFlow value in grams_per_second.
        :type grams_per_second: float
        :return: A new instance of MassFlow.
        :rtype: MassFlow
        """
        return MassFlow(grams_per_second, MassFlowUnits.GramPerSecond)

    
    @staticmethod
    def from_grams_per_day(grams_per_day: float):
        """
        Create a new instance of MassFlow from a value in grams_per_day.

        

        :param meters: The MassFlow value in grams_per_day.
        :type grams_per_day: float
        :return: A new instance of MassFlow.
        :rtype: MassFlow
        """
        return MassFlow(grams_per_day, MassFlowUnits.GramPerDay)

    
    @staticmethod
    def from_grams_per_hour(grams_per_hour: float):
        """
        Create a new instance of MassFlow from a value in grams_per_hour.

        

        :param meters: The MassFlow value in grams_per_hour.
        :type grams_per_hour: float
        :return: A new instance of MassFlow.
        :rtype: MassFlow
        """
        return MassFlow(grams_per_hour, MassFlowUnits.GramPerHour)

    
    @staticmethod
    def from_kilograms_per_hour(kilograms_per_hour: float):
        """
        Create a new instance of MassFlow from a value in kilograms_per_hour.

        

        :param meters: The MassFlow value in kilograms_per_hour.
        :type kilograms_per_hour: float
        :return: A new instance of MassFlow.
        :rtype: MassFlow
        """
        return MassFlow(kilograms_per_hour, MassFlowUnits.KilogramPerHour)

    
    @staticmethod
    def from_kilograms_per_minute(kilograms_per_minute: float):
        """
        Create a new instance of MassFlow from a value in kilograms_per_minute.

        

        :param meters: The MassFlow value in kilograms_per_minute.
        :type kilograms_per_minute: float
        :return: A new instance of MassFlow.
        :rtype: MassFlow
        """
        return MassFlow(kilograms_per_minute, MassFlowUnits.KilogramPerMinute)

    
    @staticmethod
    def from_tonnes_per_hour(tonnes_per_hour: float):
        """
        Create a new instance of MassFlow from a value in tonnes_per_hour.

        

        :param meters: The MassFlow value in tonnes_per_hour.
        :type tonnes_per_hour: float
        :return: A new instance of MassFlow.
        :rtype: MassFlow
        """
        return MassFlow(tonnes_per_hour, MassFlowUnits.TonnePerHour)

    
    @staticmethod
    def from_pounds_per_day(pounds_per_day: float):
        """
        Create a new instance of MassFlow from a value in pounds_per_day.

        

        :param meters: The MassFlow value in pounds_per_day.
        :type pounds_per_day: float
        :return: A new instance of MassFlow.
        :rtype: MassFlow
        """
        return MassFlow(pounds_per_day, MassFlowUnits.PoundPerDay)

    
    @staticmethod
    def from_pounds_per_hour(pounds_per_hour: float):
        """
        Create a new instance of MassFlow from a value in pounds_per_hour.

        

        :param meters: The MassFlow value in pounds_per_hour.
        :type pounds_per_hour: float
        :return: A new instance of MassFlow.
        :rtype: MassFlow
        """
        return MassFlow(pounds_per_hour, MassFlowUnits.PoundPerHour)

    
    @staticmethod
    def from_pounds_per_minute(pounds_per_minute: float):
        """
        Create a new instance of MassFlow from a value in pounds_per_minute.

        

        :param meters: The MassFlow value in pounds_per_minute.
        :type pounds_per_minute: float
        :return: A new instance of MassFlow.
        :rtype: MassFlow
        """
        return MassFlow(pounds_per_minute, MassFlowUnits.PoundPerMinute)

    
    @staticmethod
    def from_pounds_per_second(pounds_per_second: float):
        """
        Create a new instance of MassFlow from a value in pounds_per_second.

        

        :param meters: The MassFlow value in pounds_per_second.
        :type pounds_per_second: float
        :return: A new instance of MassFlow.
        :rtype: MassFlow
        """
        return MassFlow(pounds_per_second, MassFlowUnits.PoundPerSecond)

    
    @staticmethod
    def from_tonnes_per_day(tonnes_per_day: float):
        """
        Create a new instance of MassFlow from a value in tonnes_per_day.

        

        :param meters: The MassFlow value in tonnes_per_day.
        :type tonnes_per_day: float
        :return: A new instance of MassFlow.
        :rtype: MassFlow
        """
        return MassFlow(tonnes_per_day, MassFlowUnits.TonnePerDay)

    
    @staticmethod
    def from_short_tons_per_hour(short_tons_per_hour: float):
        """
        Create a new instance of MassFlow from a value in short_tons_per_hour.

        

        :param meters: The MassFlow value in short_tons_per_hour.
        :type short_tons_per_hour: float
        :return: A new instance of MassFlow.
        :rtype: MassFlow
        """
        return MassFlow(short_tons_per_hour, MassFlowUnits.ShortTonPerHour)

    
    @staticmethod
    def from_nanograms_per_second(nanograms_per_second: float):
        """
        Create a new instance of MassFlow from a value in nanograms_per_second.

        

        :param meters: The MassFlow value in nanograms_per_second.
        :type nanograms_per_second: float
        :return: A new instance of MassFlow.
        :rtype: MassFlow
        """
        return MassFlow(nanograms_per_second, MassFlowUnits.NanogramPerSecond)

    
    @staticmethod
    def from_micrograms_per_second(micrograms_per_second: float):
        """
        Create a new instance of MassFlow from a value in micrograms_per_second.

        

        :param meters: The MassFlow value in micrograms_per_second.
        :type micrograms_per_second: float
        :return: A new instance of MassFlow.
        :rtype: MassFlow
        """
        return MassFlow(micrograms_per_second, MassFlowUnits.MicrogramPerSecond)

    
    @staticmethod
    def from_milligrams_per_second(milligrams_per_second: float):
        """
        Create a new instance of MassFlow from a value in milligrams_per_second.

        

        :param meters: The MassFlow value in milligrams_per_second.
        :type milligrams_per_second: float
        :return: A new instance of MassFlow.
        :rtype: MassFlow
        """
        return MassFlow(milligrams_per_second, MassFlowUnits.MilligramPerSecond)

    
    @staticmethod
    def from_centigrams_per_second(centigrams_per_second: float):
        """
        Create a new instance of MassFlow from a value in centigrams_per_second.

        

        :param meters: The MassFlow value in centigrams_per_second.
        :type centigrams_per_second: float
        :return: A new instance of MassFlow.
        :rtype: MassFlow
        """
        return MassFlow(centigrams_per_second, MassFlowUnits.CentigramPerSecond)

    
    @staticmethod
    def from_decigrams_per_second(decigrams_per_second: float):
        """
        Create a new instance of MassFlow from a value in decigrams_per_second.

        

        :param meters: The MassFlow value in decigrams_per_second.
        :type decigrams_per_second: float
        :return: A new instance of MassFlow.
        :rtype: MassFlow
        """
        return MassFlow(decigrams_per_second, MassFlowUnits.DecigramPerSecond)

    
    @staticmethod
    def from_decagrams_per_second(decagrams_per_second: float):
        """
        Create a new instance of MassFlow from a value in decagrams_per_second.

        

        :param meters: The MassFlow value in decagrams_per_second.
        :type decagrams_per_second: float
        :return: A new instance of MassFlow.
        :rtype: MassFlow
        """
        return MassFlow(decagrams_per_second, MassFlowUnits.DecagramPerSecond)

    
    @staticmethod
    def from_hectograms_per_second(hectograms_per_second: float):
        """
        Create a new instance of MassFlow from a value in hectograms_per_second.

        

        :param meters: The MassFlow value in hectograms_per_second.
        :type hectograms_per_second: float
        :return: A new instance of MassFlow.
        :rtype: MassFlow
        """
        return MassFlow(hectograms_per_second, MassFlowUnits.HectogramPerSecond)

    
    @staticmethod
    def from_kilograms_per_second(kilograms_per_second: float):
        """
        Create a new instance of MassFlow from a value in kilograms_per_second.

        

        :param meters: The MassFlow value in kilograms_per_second.
        :type kilograms_per_second: float
        :return: A new instance of MassFlow.
        :rtype: MassFlow
        """
        return MassFlow(kilograms_per_second, MassFlowUnits.KilogramPerSecond)

    
    @staticmethod
    def from_nanograms_per_day(nanograms_per_day: float):
        """
        Create a new instance of MassFlow from a value in nanograms_per_day.

        

        :param meters: The MassFlow value in nanograms_per_day.
        :type nanograms_per_day: float
        :return: A new instance of MassFlow.
        :rtype: MassFlow
        """
        return MassFlow(nanograms_per_day, MassFlowUnits.NanogramPerDay)

    
    @staticmethod
    def from_micrograms_per_day(micrograms_per_day: float):
        """
        Create a new instance of MassFlow from a value in micrograms_per_day.

        

        :param meters: The MassFlow value in micrograms_per_day.
        :type micrograms_per_day: float
        :return: A new instance of MassFlow.
        :rtype: MassFlow
        """
        return MassFlow(micrograms_per_day, MassFlowUnits.MicrogramPerDay)

    
    @staticmethod
    def from_milligrams_per_day(milligrams_per_day: float):
        """
        Create a new instance of MassFlow from a value in milligrams_per_day.

        

        :param meters: The MassFlow value in milligrams_per_day.
        :type milligrams_per_day: float
        :return: A new instance of MassFlow.
        :rtype: MassFlow
        """
        return MassFlow(milligrams_per_day, MassFlowUnits.MilligramPerDay)

    
    @staticmethod
    def from_centigrams_per_day(centigrams_per_day: float):
        """
        Create a new instance of MassFlow from a value in centigrams_per_day.

        

        :param meters: The MassFlow value in centigrams_per_day.
        :type centigrams_per_day: float
        :return: A new instance of MassFlow.
        :rtype: MassFlow
        """
        return MassFlow(centigrams_per_day, MassFlowUnits.CentigramPerDay)

    
    @staticmethod
    def from_decigrams_per_day(decigrams_per_day: float):
        """
        Create a new instance of MassFlow from a value in decigrams_per_day.

        

        :param meters: The MassFlow value in decigrams_per_day.
        :type decigrams_per_day: float
        :return: A new instance of MassFlow.
        :rtype: MassFlow
        """
        return MassFlow(decigrams_per_day, MassFlowUnits.DecigramPerDay)

    
    @staticmethod
    def from_decagrams_per_day(decagrams_per_day: float):
        """
        Create a new instance of MassFlow from a value in decagrams_per_day.

        

        :param meters: The MassFlow value in decagrams_per_day.
        :type decagrams_per_day: float
        :return: A new instance of MassFlow.
        :rtype: MassFlow
        """
        return MassFlow(decagrams_per_day, MassFlowUnits.DecagramPerDay)

    
    @staticmethod
    def from_hectograms_per_day(hectograms_per_day: float):
        """
        Create a new instance of MassFlow from a value in hectograms_per_day.

        

        :param meters: The MassFlow value in hectograms_per_day.
        :type hectograms_per_day: float
        :return: A new instance of MassFlow.
        :rtype: MassFlow
        """
        return MassFlow(hectograms_per_day, MassFlowUnits.HectogramPerDay)

    
    @staticmethod
    def from_kilograms_per_day(kilograms_per_day: float):
        """
        Create a new instance of MassFlow from a value in kilograms_per_day.

        

        :param meters: The MassFlow value in kilograms_per_day.
        :type kilograms_per_day: float
        :return: A new instance of MassFlow.
        :rtype: MassFlow
        """
        return MassFlow(kilograms_per_day, MassFlowUnits.KilogramPerDay)

    
    @staticmethod
    def from_megagrams_per_day(megagrams_per_day: float):
        """
        Create a new instance of MassFlow from a value in megagrams_per_day.

        

        :param meters: The MassFlow value in megagrams_per_day.
        :type megagrams_per_day: float
        :return: A new instance of MassFlow.
        :rtype: MassFlow
        """
        return MassFlow(megagrams_per_day, MassFlowUnits.MegagramPerDay)

    
    @staticmethod
    def from_megapounds_per_day(megapounds_per_day: float):
        """
        Create a new instance of MassFlow from a value in megapounds_per_day.

        

        :param meters: The MassFlow value in megapounds_per_day.
        :type megapounds_per_day: float
        :return: A new instance of MassFlow.
        :rtype: MassFlow
        """
        return MassFlow(megapounds_per_day, MassFlowUnits.MegapoundPerDay)

    
    @staticmethod
    def from_megapounds_per_hour(megapounds_per_hour: float):
        """
        Create a new instance of MassFlow from a value in megapounds_per_hour.

        

        :param meters: The MassFlow value in megapounds_per_hour.
        :type megapounds_per_hour: float
        :return: A new instance of MassFlow.
        :rtype: MassFlow
        """
        return MassFlow(megapounds_per_hour, MassFlowUnits.MegapoundPerHour)

    
    @staticmethod
    def from_megapounds_per_minute(megapounds_per_minute: float):
        """
        Create a new instance of MassFlow from a value in megapounds_per_minute.

        

        :param meters: The MassFlow value in megapounds_per_minute.
        :type megapounds_per_minute: float
        :return: A new instance of MassFlow.
        :rtype: MassFlow
        """
        return MassFlow(megapounds_per_minute, MassFlowUnits.MegapoundPerMinute)

    
    @staticmethod
    def from_megapounds_per_second(megapounds_per_second: float):
        """
        Create a new instance of MassFlow from a value in megapounds_per_second.

        

        :param meters: The MassFlow value in megapounds_per_second.
        :type megapounds_per_second: float
        :return: A new instance of MassFlow.
        :rtype: MassFlow
        """
        return MassFlow(megapounds_per_second, MassFlowUnits.MegapoundPerSecond)

    
    @property
    def grams_per_second(self) -> float:
        """
        
        """
        if self.__grams_per_second != None:
            return self.__grams_per_second
        self.__grams_per_second = self.__convert_from_base(MassFlowUnits.GramPerSecond)
        return self.__grams_per_second

    
    @property
    def grams_per_day(self) -> float:
        """
        
        """
        if self.__grams_per_day != None:
            return self.__grams_per_day
        self.__grams_per_day = self.__convert_from_base(MassFlowUnits.GramPerDay)
        return self.__grams_per_day

    
    @property
    def grams_per_hour(self) -> float:
        """
        
        """
        if self.__grams_per_hour != None:
            return self.__grams_per_hour
        self.__grams_per_hour = self.__convert_from_base(MassFlowUnits.GramPerHour)
        return self.__grams_per_hour

    
    @property
    def kilograms_per_hour(self) -> float:
        """
        
        """
        if self.__kilograms_per_hour != None:
            return self.__kilograms_per_hour
        self.__kilograms_per_hour = self.__convert_from_base(MassFlowUnits.KilogramPerHour)
        return self.__kilograms_per_hour

    
    @property
    def kilograms_per_minute(self) -> float:
        """
        
        """
        if self.__kilograms_per_minute != None:
            return self.__kilograms_per_minute
        self.__kilograms_per_minute = self.__convert_from_base(MassFlowUnits.KilogramPerMinute)
        return self.__kilograms_per_minute

    
    @property
    def tonnes_per_hour(self) -> float:
        """
        
        """
        if self.__tonnes_per_hour != None:
            return self.__tonnes_per_hour
        self.__tonnes_per_hour = self.__convert_from_base(MassFlowUnits.TonnePerHour)
        return self.__tonnes_per_hour

    
    @property
    def pounds_per_day(self) -> float:
        """
        
        """
        if self.__pounds_per_day != None:
            return self.__pounds_per_day
        self.__pounds_per_day = self.__convert_from_base(MassFlowUnits.PoundPerDay)
        return self.__pounds_per_day

    
    @property
    def pounds_per_hour(self) -> float:
        """
        
        """
        if self.__pounds_per_hour != None:
            return self.__pounds_per_hour
        self.__pounds_per_hour = self.__convert_from_base(MassFlowUnits.PoundPerHour)
        return self.__pounds_per_hour

    
    @property
    def pounds_per_minute(self) -> float:
        """
        
        """
        if self.__pounds_per_minute != None:
            return self.__pounds_per_minute
        self.__pounds_per_minute = self.__convert_from_base(MassFlowUnits.PoundPerMinute)
        return self.__pounds_per_minute

    
    @property
    def pounds_per_second(self) -> float:
        """
        
        """
        if self.__pounds_per_second != None:
            return self.__pounds_per_second
        self.__pounds_per_second = self.__convert_from_base(MassFlowUnits.PoundPerSecond)
        return self.__pounds_per_second

    
    @property
    def tonnes_per_day(self) -> float:
        """
        
        """
        if self.__tonnes_per_day != None:
            return self.__tonnes_per_day
        self.__tonnes_per_day = self.__convert_from_base(MassFlowUnits.TonnePerDay)
        return self.__tonnes_per_day

    
    @property
    def short_tons_per_hour(self) -> float:
        """
        
        """
        if self.__short_tons_per_hour != None:
            return self.__short_tons_per_hour
        self.__short_tons_per_hour = self.__convert_from_base(MassFlowUnits.ShortTonPerHour)
        return self.__short_tons_per_hour

    
    @property
    def nanograms_per_second(self) -> float:
        """
        
        """
        if self.__nanograms_per_second != None:
            return self.__nanograms_per_second
        self.__nanograms_per_second = self.__convert_from_base(MassFlowUnits.NanogramPerSecond)
        return self.__nanograms_per_second

    
    @property
    def micrograms_per_second(self) -> float:
        """
        
        """
        if self.__micrograms_per_second != None:
            return self.__micrograms_per_second
        self.__micrograms_per_second = self.__convert_from_base(MassFlowUnits.MicrogramPerSecond)
        return self.__micrograms_per_second

    
    @property
    def milligrams_per_second(self) -> float:
        """
        
        """
        if self.__milligrams_per_second != None:
            return self.__milligrams_per_second
        self.__milligrams_per_second = self.__convert_from_base(MassFlowUnits.MilligramPerSecond)
        return self.__milligrams_per_second

    
    @property
    def centigrams_per_second(self) -> float:
        """
        
        """
        if self.__centigrams_per_second != None:
            return self.__centigrams_per_second
        self.__centigrams_per_second = self.__convert_from_base(MassFlowUnits.CentigramPerSecond)
        return self.__centigrams_per_second

    
    @property
    def decigrams_per_second(self) -> float:
        """
        
        """
        if self.__decigrams_per_second != None:
            return self.__decigrams_per_second
        self.__decigrams_per_second = self.__convert_from_base(MassFlowUnits.DecigramPerSecond)
        return self.__decigrams_per_second

    
    @property
    def decagrams_per_second(self) -> float:
        """
        
        """
        if self.__decagrams_per_second != None:
            return self.__decagrams_per_second
        self.__decagrams_per_second = self.__convert_from_base(MassFlowUnits.DecagramPerSecond)
        return self.__decagrams_per_second

    
    @property
    def hectograms_per_second(self) -> float:
        """
        
        """
        if self.__hectograms_per_second != None:
            return self.__hectograms_per_second
        self.__hectograms_per_second = self.__convert_from_base(MassFlowUnits.HectogramPerSecond)
        return self.__hectograms_per_second

    
    @property
    def kilograms_per_second(self) -> float:
        """
        
        """
        if self.__kilograms_per_second != None:
            return self.__kilograms_per_second
        self.__kilograms_per_second = self.__convert_from_base(MassFlowUnits.KilogramPerSecond)
        return self.__kilograms_per_second

    
    @property
    def nanograms_per_day(self) -> float:
        """
        
        """
        if self.__nanograms_per_day != None:
            return self.__nanograms_per_day
        self.__nanograms_per_day = self.__convert_from_base(MassFlowUnits.NanogramPerDay)
        return self.__nanograms_per_day

    
    @property
    def micrograms_per_day(self) -> float:
        """
        
        """
        if self.__micrograms_per_day != None:
            return self.__micrograms_per_day
        self.__micrograms_per_day = self.__convert_from_base(MassFlowUnits.MicrogramPerDay)
        return self.__micrograms_per_day

    
    @property
    def milligrams_per_day(self) -> float:
        """
        
        """
        if self.__milligrams_per_day != None:
            return self.__milligrams_per_day
        self.__milligrams_per_day = self.__convert_from_base(MassFlowUnits.MilligramPerDay)
        return self.__milligrams_per_day

    
    @property
    def centigrams_per_day(self) -> float:
        """
        
        """
        if self.__centigrams_per_day != None:
            return self.__centigrams_per_day
        self.__centigrams_per_day = self.__convert_from_base(MassFlowUnits.CentigramPerDay)
        return self.__centigrams_per_day

    
    @property
    def decigrams_per_day(self) -> float:
        """
        
        """
        if self.__decigrams_per_day != None:
            return self.__decigrams_per_day
        self.__decigrams_per_day = self.__convert_from_base(MassFlowUnits.DecigramPerDay)
        return self.__decigrams_per_day

    
    @property
    def decagrams_per_day(self) -> float:
        """
        
        """
        if self.__decagrams_per_day != None:
            return self.__decagrams_per_day
        self.__decagrams_per_day = self.__convert_from_base(MassFlowUnits.DecagramPerDay)
        return self.__decagrams_per_day

    
    @property
    def hectograms_per_day(self) -> float:
        """
        
        """
        if self.__hectograms_per_day != None:
            return self.__hectograms_per_day
        self.__hectograms_per_day = self.__convert_from_base(MassFlowUnits.HectogramPerDay)
        return self.__hectograms_per_day

    
    @property
    def kilograms_per_day(self) -> float:
        """
        
        """
        if self.__kilograms_per_day != None:
            return self.__kilograms_per_day
        self.__kilograms_per_day = self.__convert_from_base(MassFlowUnits.KilogramPerDay)
        return self.__kilograms_per_day

    
    @property
    def megagrams_per_day(self) -> float:
        """
        
        """
        if self.__megagrams_per_day != None:
            return self.__megagrams_per_day
        self.__megagrams_per_day = self.__convert_from_base(MassFlowUnits.MegagramPerDay)
        return self.__megagrams_per_day

    
    @property
    def megapounds_per_day(self) -> float:
        """
        
        """
        if self.__megapounds_per_day != None:
            return self.__megapounds_per_day
        self.__megapounds_per_day = self.__convert_from_base(MassFlowUnits.MegapoundPerDay)
        return self.__megapounds_per_day

    
    @property
    def megapounds_per_hour(self) -> float:
        """
        
        """
        if self.__megapounds_per_hour != None:
            return self.__megapounds_per_hour
        self.__megapounds_per_hour = self.__convert_from_base(MassFlowUnits.MegapoundPerHour)
        return self.__megapounds_per_hour

    
    @property
    def megapounds_per_minute(self) -> float:
        """
        
        """
        if self.__megapounds_per_minute != None:
            return self.__megapounds_per_minute
        self.__megapounds_per_minute = self.__convert_from_base(MassFlowUnits.MegapoundPerMinute)
        return self.__megapounds_per_minute

    
    @property
    def megapounds_per_second(self) -> float:
        """
        
        """
        if self.__megapounds_per_second != None:
            return self.__megapounds_per_second
        self.__megapounds_per_second = self.__convert_from_base(MassFlowUnits.MegapoundPerSecond)
        return self.__megapounds_per_second

    
    def to_string(self, unit: MassFlowUnits = MassFlowUnits.GramPerSecond, fractional_digits: int = None) -> str:
        """
        Format the MassFlow to a string.
        
        Note: the default format for MassFlow is GramPerSecond.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the MassFlow. Default is 'GramPerSecond'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == MassFlowUnits.GramPerSecond:
            return f"""{super()._truncate_fraction_digits(self.grams_per_second, fractional_digits)} g/s"""
        
        if unit == MassFlowUnits.GramPerDay:
            return f"""{super()._truncate_fraction_digits(self.grams_per_day, fractional_digits)} g/d"""
        
        if unit == MassFlowUnits.GramPerHour:
            return f"""{super()._truncate_fraction_digits(self.grams_per_hour, fractional_digits)} g/h"""
        
        if unit == MassFlowUnits.KilogramPerHour:
            return f"""{super()._truncate_fraction_digits(self.kilograms_per_hour, fractional_digits)} kg/h"""
        
        if unit == MassFlowUnits.KilogramPerMinute:
            return f"""{super()._truncate_fraction_digits(self.kilograms_per_minute, fractional_digits)} kg/min"""
        
        if unit == MassFlowUnits.TonnePerHour:
            return f"""{super()._truncate_fraction_digits(self.tonnes_per_hour, fractional_digits)} t/h"""
        
        if unit == MassFlowUnits.PoundPerDay:
            return f"""{super()._truncate_fraction_digits(self.pounds_per_day, fractional_digits)} lb/d"""
        
        if unit == MassFlowUnits.PoundPerHour:
            return f"""{super()._truncate_fraction_digits(self.pounds_per_hour, fractional_digits)} lb/h"""
        
        if unit == MassFlowUnits.PoundPerMinute:
            return f"""{super()._truncate_fraction_digits(self.pounds_per_minute, fractional_digits)} lb/min"""
        
        if unit == MassFlowUnits.PoundPerSecond:
            return f"""{super()._truncate_fraction_digits(self.pounds_per_second, fractional_digits)} lb/s"""
        
        if unit == MassFlowUnits.TonnePerDay:
            return f"""{super()._truncate_fraction_digits(self.tonnes_per_day, fractional_digits)} t/d"""
        
        if unit == MassFlowUnits.ShortTonPerHour:
            return f"""{super()._truncate_fraction_digits(self.short_tons_per_hour, fractional_digits)} short tn/h"""
        
        if unit == MassFlowUnits.NanogramPerSecond:
            return f"""{super()._truncate_fraction_digits(self.nanograms_per_second, fractional_digits)} ng/s"""
        
        if unit == MassFlowUnits.MicrogramPerSecond:
            return f"""{super()._truncate_fraction_digits(self.micrograms_per_second, fractional_digits)} μg/s"""
        
        if unit == MassFlowUnits.MilligramPerSecond:
            return f"""{super()._truncate_fraction_digits(self.milligrams_per_second, fractional_digits)} mg/s"""
        
        if unit == MassFlowUnits.CentigramPerSecond:
            return f"""{super()._truncate_fraction_digits(self.centigrams_per_second, fractional_digits)} cg/s"""
        
        if unit == MassFlowUnits.DecigramPerSecond:
            return f"""{super()._truncate_fraction_digits(self.decigrams_per_second, fractional_digits)} dg/s"""
        
        if unit == MassFlowUnits.DecagramPerSecond:
            return f"""{super()._truncate_fraction_digits(self.decagrams_per_second, fractional_digits)} dag/s"""
        
        if unit == MassFlowUnits.HectogramPerSecond:
            return f"""{super()._truncate_fraction_digits(self.hectograms_per_second, fractional_digits)} hg/s"""
        
        if unit == MassFlowUnits.KilogramPerSecond:
            return f"""{super()._truncate_fraction_digits(self.kilograms_per_second, fractional_digits)} kg/s"""
        
        if unit == MassFlowUnits.NanogramPerDay:
            return f"""{super()._truncate_fraction_digits(self.nanograms_per_day, fractional_digits)} ng/d"""
        
        if unit == MassFlowUnits.MicrogramPerDay:
            return f"""{super()._truncate_fraction_digits(self.micrograms_per_day, fractional_digits)} μg/d"""
        
        if unit == MassFlowUnits.MilligramPerDay:
            return f"""{super()._truncate_fraction_digits(self.milligrams_per_day, fractional_digits)} mg/d"""
        
        if unit == MassFlowUnits.CentigramPerDay:
            return f"""{super()._truncate_fraction_digits(self.centigrams_per_day, fractional_digits)} cg/d"""
        
        if unit == MassFlowUnits.DecigramPerDay:
            return f"""{super()._truncate_fraction_digits(self.decigrams_per_day, fractional_digits)} dg/d"""
        
        if unit == MassFlowUnits.DecagramPerDay:
            return f"""{super()._truncate_fraction_digits(self.decagrams_per_day, fractional_digits)} dag/d"""
        
        if unit == MassFlowUnits.HectogramPerDay:
            return f"""{super()._truncate_fraction_digits(self.hectograms_per_day, fractional_digits)} hg/d"""
        
        if unit == MassFlowUnits.KilogramPerDay:
            return f"""{super()._truncate_fraction_digits(self.kilograms_per_day, fractional_digits)} kg/d"""
        
        if unit == MassFlowUnits.MegagramPerDay:
            return f"""{super()._truncate_fraction_digits(self.megagrams_per_day, fractional_digits)} Mg/d"""
        
        if unit == MassFlowUnits.MegapoundPerDay:
            return f"""{super()._truncate_fraction_digits(self.megapounds_per_day, fractional_digits)} Mlb/d"""
        
        if unit == MassFlowUnits.MegapoundPerHour:
            return f"""{super()._truncate_fraction_digits(self.megapounds_per_hour, fractional_digits)} Mlb/h"""
        
        if unit == MassFlowUnits.MegapoundPerMinute:
            return f"""{super()._truncate_fraction_digits(self.megapounds_per_minute, fractional_digits)} Mlb/min"""
        
        if unit == MassFlowUnits.MegapoundPerSecond:
            return f"""{super()._truncate_fraction_digits(self.megapounds_per_second, fractional_digits)} Mlb/s"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: MassFlowUnits = MassFlowUnits.GramPerSecond) -> str:
        """
        Get MassFlow unit abbreviation.
        Note! the default abbreviation for MassFlow is GramPerSecond.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == MassFlowUnits.GramPerSecond:
            return """g/s"""
        
        if unit_abbreviation == MassFlowUnits.GramPerDay:
            return """g/d"""
        
        if unit_abbreviation == MassFlowUnits.GramPerHour:
            return """g/h"""
        
        if unit_abbreviation == MassFlowUnits.KilogramPerHour:
            return """kg/h"""
        
        if unit_abbreviation == MassFlowUnits.KilogramPerMinute:
            return """kg/min"""
        
        if unit_abbreviation == MassFlowUnits.TonnePerHour:
            return """t/h"""
        
        if unit_abbreviation == MassFlowUnits.PoundPerDay:
            return """lb/d"""
        
        if unit_abbreviation == MassFlowUnits.PoundPerHour:
            return """lb/h"""
        
        if unit_abbreviation == MassFlowUnits.PoundPerMinute:
            return """lb/min"""
        
        if unit_abbreviation == MassFlowUnits.PoundPerSecond:
            return """lb/s"""
        
        if unit_abbreviation == MassFlowUnits.TonnePerDay:
            return """t/d"""
        
        if unit_abbreviation == MassFlowUnits.ShortTonPerHour:
            return """short tn/h"""
        
        if unit_abbreviation == MassFlowUnits.NanogramPerSecond:
            return """ng/s"""
        
        if unit_abbreviation == MassFlowUnits.MicrogramPerSecond:
            return """μg/s"""
        
        if unit_abbreviation == MassFlowUnits.MilligramPerSecond:
            return """mg/s"""
        
        if unit_abbreviation == MassFlowUnits.CentigramPerSecond:
            return """cg/s"""
        
        if unit_abbreviation == MassFlowUnits.DecigramPerSecond:
            return """dg/s"""
        
        if unit_abbreviation == MassFlowUnits.DecagramPerSecond:
            return """dag/s"""
        
        if unit_abbreviation == MassFlowUnits.HectogramPerSecond:
            return """hg/s"""
        
        if unit_abbreviation == MassFlowUnits.KilogramPerSecond:
            return """kg/s"""
        
        if unit_abbreviation == MassFlowUnits.NanogramPerDay:
            return """ng/d"""
        
        if unit_abbreviation == MassFlowUnits.MicrogramPerDay:
            return """μg/d"""
        
        if unit_abbreviation == MassFlowUnits.MilligramPerDay:
            return """mg/d"""
        
        if unit_abbreviation == MassFlowUnits.CentigramPerDay:
            return """cg/d"""
        
        if unit_abbreviation == MassFlowUnits.DecigramPerDay:
            return """dg/d"""
        
        if unit_abbreviation == MassFlowUnits.DecagramPerDay:
            return """dag/d"""
        
        if unit_abbreviation == MassFlowUnits.HectogramPerDay:
            return """hg/d"""
        
        if unit_abbreviation == MassFlowUnits.KilogramPerDay:
            return """kg/d"""
        
        if unit_abbreviation == MassFlowUnits.MegagramPerDay:
            return """Mg/d"""
        
        if unit_abbreviation == MassFlowUnits.MegapoundPerDay:
            return """Mlb/d"""
        
        if unit_abbreviation == MassFlowUnits.MegapoundPerHour:
            return """Mlb/h"""
        
        if unit_abbreviation == MassFlowUnits.MegapoundPerMinute:
            return """Mlb/min"""
        
        if unit_abbreviation == MassFlowUnits.MegapoundPerSecond:
            return """Mlb/s"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class ThermalConductivityUnits(Enum):
        """
            ThermalConductivityUnits enumeration
        """
        
        WattPerMeterKelvin = 'WattPerMeterKelvin'
        """
            
        """
        
        BtuPerHourFootFahrenheit = 'BtuPerHourFootFahrenheit'
        """
            
        """
        

class ThermalConductivityDto:
    """
    A DTO representation of a ThermalConductivity

    Attributes:
        value (float): The value of the ThermalConductivity.
        unit (ThermalConductivityUnits): The specific unit that the ThermalConductivity value is representing.
    """

    def __init__(self, value: float, unit: ThermalConductivityUnits):
        """
        Create a new DTO representation of a ThermalConductivity

        Parameters:
            value (float): The value of the ThermalConductivity.
            unit (ThermalConductivityUnits): The specific unit that the ThermalConductivity value is representing.
        """
        self.value: float = value
        """
        The value of the ThermalConductivity
        """
        self.unit: ThermalConductivityUnits = unit
        """
        The specific unit that the ThermalConductivity value is representing
        """

    def to_json(self):
        """
        Get a ThermalConductivity DTO JSON object representing the current unit.

        :return: JSON object represents ThermalConductivity DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "WattPerMeterKelvin"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of ThermalConductivity DTO from a json representation.

        :param data: The ThermalConductivity DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "WattPerMeterKelvin"}
        :return: A new instance of ThermalConductivityDto.
        :rtype: ThermalConductivityDto
        """
        return ThermalConductivityDto(value=data["value"], unit=ThermalConductivityUnits(data["unit"]))


class ThermalConductivity(AbstractMeasure):
    """
    Thermal conductivity is the property of a material to conduct heat.

    Args:
        value (float): The value.
        from_unit (ThermalConductivityUnits): The ThermalConductivity unit to create from, The default unit is WattPerMeterKelvin
    """
    def __init__(self, value: float, from_unit: ThermalConductivityUnits = ThermalConductivityUnits.WattPerMeterKelvin):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__watts_per_meter_kelvin = None
        
        self.__btus_per_hour_foot_fahrenheit = None
        

    def convert(self, unit: ThermalConductivityUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: ThermalConductivityUnits = ThermalConductivityUnits.WattPerMeterKelvin) -> ThermalConductivityDto:
        """
        Get a new instance of ThermalConductivity DTO representing the current unit.

        :param hold_in_unit: The specific ThermalConductivity unit to store the ThermalConductivity value in the DTO representation.
        :type hold_in_unit: ThermalConductivityUnits
        :return: A new instance of ThermalConductivityDto.
        :rtype: ThermalConductivityDto
        """
        return ThermalConductivityDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: ThermalConductivityUnits = ThermalConductivityUnits.WattPerMeterKelvin):
        """
        Get a ThermalConductivity DTO JSON object representing the current unit.

        :param hold_in_unit: The specific ThermalConductivity unit to store the ThermalConductivity value in the DTO representation.
        :type hold_in_unit: ThermalConductivityUnits
        :return: JSON object represents ThermalConductivity DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "WattPerMeterKelvin"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(thermal_conductivity_dto: ThermalConductivityDto):
        """
        Obtain a new instance of ThermalConductivity from a DTO unit object.

        :param thermal_conductivity_dto: The ThermalConductivity DTO representation.
        :type thermal_conductivity_dto: ThermalConductivityDto
        :return: A new instance of ThermalConductivity.
        :rtype: ThermalConductivity
        """
        return ThermalConductivity(thermal_conductivity_dto.value, thermal_conductivity_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of ThermalConductivity from a DTO unit json representation.

        :param data: The ThermalConductivity DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "WattPerMeterKelvin"}
        :return: A new instance of ThermalConductivity.
        :rtype: ThermalConductivity
        """
        return ThermalConductivity.from_dto(ThermalConductivityDto.from_json(data))

    def __convert_from_base(self, from_unit: ThermalConductivityUnits) -> float:
        value = self._value
        
        if from_unit == ThermalConductivityUnits.WattPerMeterKelvin:
            return (value)
        
        if from_unit == ThermalConductivityUnits.BtuPerHourFootFahrenheit:
            return (value / 1.73073467)
        
        return None


    def __convert_to_base(self, value: float, to_unit: ThermalConductivityUnits) -> float:
        
        if to_unit == ThermalConductivityUnits.WattPerMeterKelvin:
            return (value)
        
        if to_unit == ThermalConductivityUnits.BtuPerHourFootFahrenheit:
            return (value * 1.73073467)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_watts_per_meter_kelvin(watts_per_meter_kelvin: float):
        """
        Create a new instance of ThermalConductivity from a value in watts_per_meter_kelvin.

        

        :param meters: The ThermalConductivity value in watts_per_meter_kelvin.
        :type watts_per_meter_kelvin: float
        :return: A new instance of ThermalConductivity.
        :rtype: ThermalConductivity
        """
        return ThermalConductivity(watts_per_meter_kelvin, ThermalConductivityUnits.WattPerMeterKelvin)

    
    @staticmethod
    def from_btus_per_hour_foot_fahrenheit(btus_per_hour_foot_fahrenheit: float):
        """
        Create a new instance of ThermalConductivity from a value in btus_per_hour_foot_fahrenheit.

        

        :param meters: The ThermalConductivity value in btus_per_hour_foot_fahrenheit.
        :type btus_per_hour_foot_fahrenheit: float
        :return: A new instance of ThermalConductivity.
        :rtype: ThermalConductivity
        """
        return ThermalConductivity(btus_per_hour_foot_fahrenheit, ThermalConductivityUnits.BtuPerHourFootFahrenheit)

    
    @property
    def watts_per_meter_kelvin(self) -> float:
        """
        
        """
        if self.__watts_per_meter_kelvin != None:
            return self.__watts_per_meter_kelvin
        self.__watts_per_meter_kelvin = self.__convert_from_base(ThermalConductivityUnits.WattPerMeterKelvin)
        return self.__watts_per_meter_kelvin

    
    @property
    def btus_per_hour_foot_fahrenheit(self) -> float:
        """
        
        """
        if self.__btus_per_hour_foot_fahrenheit != None:
            return self.__btus_per_hour_foot_fahrenheit
        self.__btus_per_hour_foot_fahrenheit = self.__convert_from_base(ThermalConductivityUnits.BtuPerHourFootFahrenheit)
        return self.__btus_per_hour_foot_fahrenheit

    
    def to_string(self, unit: ThermalConductivityUnits = ThermalConductivityUnits.WattPerMeterKelvin, fractional_digits: int = None) -> str:
        """
        Format the ThermalConductivity to a string.
        
        Note: the default format for ThermalConductivity is WattPerMeterKelvin.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the ThermalConductivity. Default is 'WattPerMeterKelvin'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == ThermalConductivityUnits.WattPerMeterKelvin:
            return f"""{super()._truncate_fraction_digits(self.watts_per_meter_kelvin, fractional_digits)} W/m·K"""
        
        if unit == ThermalConductivityUnits.BtuPerHourFootFahrenheit:
            return f"""{super()._truncate_fraction_digits(self.btus_per_hour_foot_fahrenheit, fractional_digits)} BTU/h·ft·°F"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: ThermalConductivityUnits = ThermalConductivityUnits.WattPerMeterKelvin) -> str:
        """
        Get ThermalConductivity unit abbreviation.
        Note! the default abbreviation for ThermalConductivity is WattPerMeterKelvin.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == ThermalConductivityUnits.WattPerMeterKelvin:
            return """W/m·K"""
        
        if unit_abbreviation == ThermalConductivityUnits.BtuPerHourFootFahrenheit:
            return """BTU/h·ft·°F"""
        